/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;
import java.util.*;

import org.homedns.dade.jcgrid.server.*;

public class GridConfig implements Cloneable {
    public final static int DEFAULT_CLIENT_PORT=10101;
    public final static int DEFAULT_WORKER_PORT=10102;
    public final static int DEFAULT_ADMIN_PORT=10103;

    private String serverAddress;

    private int serverClientPort;
    private int serverWorkerPort;
    private int serverAdminPort;

    private boolean useSecureConnection;
    private boolean useCompression;
    private boolean useVFS;
    private String serverPassword;
    
    public GridConfig(Properties prop) {
        serverAddress=prop.getProperty("grid.server.hostname","localhost");

        serverClientPort=Integer.parseInt(prop.getProperty(
            "grid.server.client.port",
            Integer.toString(GridConfig.DEFAULT_CLIENT_PORT)));
        serverWorkerPort=Integer.parseInt(prop.getProperty(
            "grid.server.worker.port",
            Integer.toString(GridConfig.DEFAULT_WORKER_PORT)));
        serverAdminPort=Integer.parseInt(prop.getProperty(
            "grid.server.admin.port",
            Integer.toString(GridConfig.DEFAULT_ADMIN_PORT)));

        useSecureConnection=Boolean.valueOf(prop.getProperty(
            "grid.server.useencryption",
            Boolean.toString(false))).booleanValue();
        useCompression=Boolean.valueOf(prop.getProperty(
            "grid.server.usecompression",
            Boolean.toString(false))).booleanValue();
        useVFS=Boolean.valueOf(prop.getProperty(
            "grid.server.usevfs",
            Boolean.toString(true))).booleanValue();
    }
    
    public GridConfig() {
        serverAddress="localhost";

        serverClientPort=GridConfig.DEFAULT_CLIENT_PORT;
        serverWorkerPort=GridConfig.DEFAULT_WORKER_PORT;
        serverAdminPort=GridConfig.DEFAULT_ADMIN_PORT;

        useSecureConnection=false;
        useCompression=false;
        useVFS=true;
    }

    public Object clone() {
        try {
            GridConfig cfg=(GridConfig)super.clone();
            cfg.serverAddress=serverAddress;

            cfg.serverClientPort=serverClientPort;
            cfg.serverWorkerPort=serverWorkerPort;
            cfg.serverAdminPort=serverAdminPort;

            cfg.useSecureConnection=useSecureConnection;
            cfg.useCompression=useCompression;
            cfg.serverPassword=serverPassword;
            cfg.useVFS=useVFS;

            return cfg;
        } catch (CloneNotSupportedException e) {
            throw new InternalError(e.toString());
        }
    }
    
    public Properties toProperties() {
        Properties prop=new Properties();

        prop.put("grid.server.hostname",serverAddress);
        prop.put("grid.server.client.port",Integer.toString(serverClientPort));
        prop.put("grid.server.worker.port",Integer.toString(serverWorkerPort));
        prop.put("grid.server.admin.port",Integer.toString(serverAdminPort));
        prop.put("grid.server.useencryption",Boolean.toString(useSecureConnection));
        prop.put("grid.server.usecompression",Boolean.toString(useCompression));
        prop.put("grid.server.usevfs",Boolean.toString(useVFS));

        return prop;
    }
    
    //------------------------------- Get & Set --------------------------------

    public String getServerAddress() {
        return serverAddress;
    }

    public void setServerAddress(String server) {
        serverAddress=server;
    }

    public int getServerClientPort() {
        return serverClientPort;
    }

    public void setServerClientPort(int port) {
        serverClientPort=port;
    }

    public int getServerWorkerPort() {
        return serverWorkerPort;
    }

    public void setServerWorkerPort(int port) {
        serverWorkerPort=port;
    }

    public int getServerAdminPort() {
        return serverAdminPort;
    }

    public void setServerAdminPort(int port) {
        serverAdminPort=port;
    }

    public String getServerPassword() {
        return serverPassword;
    }

    public void setServerPassword(String pwd) {
        serverPassword=pwd;
    }

    public boolean getUseSecureConnection() {
        return useSecureConnection;
    }

    public void setUseSecureConnection(boolean use) {
        useSecureConnection=use;
    }

    public boolean getUseCompression() {
        return useCompression;
    }

    public void setUseCompression(boolean use) {
        useCompression=use;
    }

    public boolean getUseVFS() {
        return useVFS;
    }

    public void setUseVFS(boolean use) {
        useVFS=use;
    }
}
