/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;
import java.net.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.server.*;
import org.homedns.dade.jcgrid.util.*;

public abstract class GridNode {
    private final static String className=GridNode.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);
    
    private GridNodeConfig config;

    public GridNode(GridNodeConfig cfg) {
        if(log.isDebugEnabled()) log.debug("Start GridNode("+cfg+")");

        config=cfg;

        if(log.isDebugEnabled()) log.debug("End GridNode()");
    }

    //----------------------------- Start & Stop -------------------------------

    public void start()  throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start start()");

        log.warn("Starting JCGrid Node...");

        GridConfig gCfg=config.getGridConfig();
        
        log.warn("  Server address: "+gCfg.getServerAddress());
        log.warn("  Server client port: "+gCfg.getServerClientPort());
        log.warn("  Server worker port: "+gCfg.getServerWorkerPort());
        log.warn("  Server admin. port: "+gCfg.getServerAdminPort());
        log.warn("  Working dir.: "+config.getWorkingDir());
        log.warn("  Use secure connection: "+(gCfg.getUseSecureConnection() ? "yes" : "no"));
        log.warn("  Node type: "+config.getType());
        log.warn("  Use compressed connection: "+(gCfg.getUseCompression() ? "yes" : "no"));

        if(logDetail.isDebugEnabled()) logDetail.debug("End start()");
    }

    public void stop() throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start stop()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End stop()");
    }

    //------------------------------- Get & Set --------------------------------

    public GridNodeConfig getNodeConfig() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getNodeConfig()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getNodeConfig("+config+")");
        
        return config;
    }

    public void setNodeConfig(GridNodeGenericConfig cfg) {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start setNodeConfig("+cfg+")");

        config=cfg;

        if(logDetail.isDebugEnabled()) logDetail.debug("End setNodeConfig()");
    }
}
