/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;
import java.util.*;

import org.homedns.dade.jcgrid.server.*;

public abstract class GridNodeConfig implements Cloneable {
    public final static String TYPE_WORKER="WORKER";
    public final static String TYPE_CLIENT="CLIENT";
    public final static String TYPE_ADMIN="ADMIN";
    public final static String TYPE_SERVER="SERVER";

    private GridConfig gCfg;

    private String nodeType;
    private String workingDir;

    public GridNodeConfig(Properties prop) {
        gCfg=new GridConfig(prop);

        nodeType=prop.getProperty("grid.node.type",TYPE_CLIENT);
        workingDir=prop.getProperty("grid.node.workingdir",
            System.getProperty("user.dir")+File.separator+"cache");
    }

    public GridNodeConfig(String type) {
        gCfg=new GridConfig();

        nodeType=type;
        workingDir=System.getProperty("user.dir")+File.separator+"cache";
    }

    public Object clone() {
        try {
            GridNodeConfig cfg=(GridNodeConfig)super.clone();
            cfg.gCfg=gCfg;

            cfg.nodeType=nodeType;
            cfg.workingDir=workingDir;

            return cfg;
        } catch (CloneNotSupportedException e) {
            throw new InternalError(e.toString());
        }
    }
    
    public Properties toProperties() {
        Properties prop=gCfg.toProperties();

        prop.put("grid.node.type",nodeType);
        prop.put("grid.node.workingdir",workingDir);

        return prop;
    }

    //------------------------------- Get & Set --------------------------------

    public GridConfig getGridConfig() {
        return gCfg;
    }

    public void setGridConfig(GridConfig cfg) {
        gCfg=cfg;
    }

    public String getWorkingDir() {
        return workingDir;
    }

    public void setWorkingDir(String dir) {
        workingDir=dir;
    }

    public String getType() {
        return nodeType;
    }

    public void setType(String type) {
        nodeType=type;
    }
}
