/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;
import java.net.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.server.*;
import org.homedns.dade.jcgrid.util.*;

public abstract class GridNodeGeneric extends GridNode {
    private final static String className=GridNodeGeneric.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private Socket serverSocket;
    private GridMessageChannel serverChannel;

    public GridNodeGeneric(GridNodeGenericConfig cfg) {
        super(cfg);

        if(log.isDebugEnabled()) log.debug("Start GridNode("+cfg+")");

        serverSocket=null;
        serverChannel=null;

        if(log.isDebugEnabled()) log.debug("End GridNode()");
    }

    //----------------------------- Start & Stop -------------------------------

    public void start()  throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start start()");
        
        super.start();

        // Connect to server

        log.warn("  Connecting to server...");

        // Selecting the right port

        GridConfig gCfg=this.getNodeConfig().getGridConfig();

        int port=gCfg.getServerClientPort();
        if(GridNodeConfig.TYPE_WORKER.equals(this.getNodeConfig().getType()))
            port=gCfg.getServerWorkerPort();
        else if(GridNodeConfig.TYPE_ADMIN.equals(this.getNodeConfig().getType()))
            port=gCfg.getServerAdminPort();
        
        if(gCfg.getUseSecureConnection()) {
            SSLSocketFactory sslFact=
                (SSLSocketFactory)SSLSocketFactory.getDefault();
            serverSocket=sslFact.createSocket(gCfg.getServerAddress(),port);
        } else 
            serverSocket=new Socket(gCfg.getServerAddress(),port);

        if(gCfg.getUseCompression())
            serverChannel=new GridMessageGZIPChannel(serverSocket);
        else
            serverChannel=new GridMessageFlatChannel(serverSocket,false);

        log.warn("  Ok");

        // Send login information

        log.warn("  Sending login information...");
        serverChannel.send(new GridMessageBeginSession(
            ((GridNodeGenericConfig)this.getNodeConfig()).
                getSessionName(),gCfg.getServerPassword()));

        log.warn("  Ok");

        if(logDetail.isDebugEnabled()) logDetail.debug("End start()");
    }

    public void stop() throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start stop()");

        try {
            serverChannel.send(new GridMessageEndSession());
        } catch(Exception ex) {
            if(log.isDebugEnabled()) log.debug("End session failed",ex);
        }

        try {
            serverChannel.close();
        } catch(Exception ex) {
            if(logDetail.isDebugEnabled()) logDetail.debug("Close server channel connection failed",ex);
        }
        
        if(logDetail.isDebugEnabled()) logDetail.debug("End stop()");
    }

    //------------------------------- Get & Set --------------------------------

    public GridMessageChannel getGridMessageChannel() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getGridMessageChannel()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getGridMessageChannel("+serverChannel+")");
        
        return serverChannel;
    }
}
