/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;
import java.util.*;

import org.homedns.dade.jcgrid.server.*;

public abstract class GridNodeGenericConfig extends GridNodeConfig
    implements Cloneable {
    private static Random rndGenerator=new Random();
    
    private static String generateSessionID() {
        final String[] alphabet={
            "A","B","C","D","E","F","G","H",
            "I","J","K","L","M","N","O","P",
            "Q","R","S","T","U","V","W","X",
            "Y","Z","a","b","c","d","e","f",
            "g","h","i","j","k","l","m","n",
            "o","p","q","r","s","t","u","v",
            "w","x","y","z","0","1","2","3",
            "4","5","6","7","8","9"
        };

        StringBuffer  sb=new StringBuffer();
        synchronized(rndGenerator) {
            for(int i=0;i<8;i++)
                sb.append(alphabet[rndGenerator.nextInt(alphabet.length)]);
        }
        
        return sb.toString();
    }
    
    private String sessionName;

    public GridNodeGenericConfig(Properties prop) {
        super(prop);

        sessionName=prop.getProperty("grid.node.session.name","none");
    }
    
    public GridNodeGenericConfig(String type) {
        super(type);

        sessionName="none";
    }

    public Object clone() {
        GridNodeGenericConfig cfg=(GridNodeGenericConfig)super.clone();

        cfg.sessionName=sessionName;

        return cfg;
    }

    public Properties toProperties() {
        Properties prop=super.toProperties();

        prop.put("grid.node.session.name",sessionName);

        return prop;
    }

    public void autoSessioName() {
        String s=System.getProperty("user.name","noname")+"_"+
            this.generateSessionID();
        sessionName=s.replaceAll("[^a-zA-Z0-9_]+","_");
    }

    //------------------------------- Get & Set --------------------------------

    public String getSessionName() {
        return sessionName;
    }

    public void setSessionName(String name) throws Exception {
        // If you change the regexp check also the server side code
        
        if(!name.matches("[a-zA-Z0-9_]+"))
            throw new Exception("Invalid session name");
        
        sessionName=name;
    }
}
