/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid;

import java.io.*;

public class WorkerStats implements Serializable {
    private static final long serialVersionUID=1L;

    public final static short STATUS_FREE=0;
    public final static short STATUS_BUSY=1;
    
    private String name;

    private short status;
    private String workingFor;
    private long workingStarted;

    private long unitDone;
    private double unitSec;

    public WorkerStats(String workerName) {
        name=workerName;
        
        status=STATUS_FREE;
        workingFor="";
        workingStarted=0;
        
        unitDone=0;
        unitSec=0.0;
    }

    public synchronized String getName() {
        return name;
    }

    public synchronized short getStatus() {
        return status;
    }

    public synchronized String getWorkingFor() {
        return workingFor;
    }

    public synchronized long getWorkingStarted() {
        return workingStarted;
    }

    public synchronized long getUnitDone() {
        return unitDone;
    }

    public synchronized double getUnitSec() {
        return unitSec;
    }

    public synchronized void setStatusFree() {
        status=STATUS_FREE;
        workingFor="";
        workingStarted=0;
    }

    public synchronized void workBegin(String sessionName) {
        status=STATUS_BUSY;
        workingFor=sessionName;
        workingStarted=System.currentTimeMillis();
    }

    public synchronized void workEnd(WorkResult wr) {
        long t=System.currentTimeMillis()-workingStarted+1;

        this.setStatusFree();

        unitDone+=wr.getUnitDone();

        if(unitDone==0)
            unitSec=1000.0*wr.getUnitDone()/t;
        else {
            double k=wr.getUnitDone()/(double)unitDone;
            unitSec=unitSec*(1.0-k)+(1000.0*wr.getUnitDone()/t)*k;
        }
    }
}
