/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.admin;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.server.*;
import org.homedns.dade.jcgrid.vfs.*;
import org.homedns.dade.jcgrid.util.*;

public class GridAdmin extends GridNodeGeneric {
    private final static String className=GridAdmin.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private vfsSession session;

    public GridAdmin() {
        super(new GridNodeAdminConfig());

        if(log.isDebugEnabled()) log.debug("Start GridAdmin()");

        session=null;
        
        super.getNodeConfig().setWorkingDir(System.getProperty("user.dir"));

        if(log.isDebugEnabled()) log.debug("End GridAdmin()");
    }

    public long ping() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start ping()");

        long res;
        long t1=System.currentTimeMillis();
        
        super.getGridMessageChannel().send(new GridMessagePing());
        GridMessage msg=super.getGridMessageChannel().recv(30*1000);

        if((msg==null) || (!(msg instanceof GridMessagePingAck)))
            res=-1;
        else
            res=System.currentTimeMillis()-t1;

        if(log.isDebugEnabled()) log.debug("End ping("+res+")");
        
        return res;
    }

    public Vector getWorkerStats() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start getWorkerStats()");

        Vector res;
        super.getGridMessageChannel().send(new GridMessageAdminGetWorkerStats());
        GridMessage msg=super.getGridMessageChannel().recv(30*1000);

        if((msg==null) || (!(msg instanceof GridMessageAdminWorkerStats)))
            res=null;
        else
            res=((GridMessageAdminWorkerStats)msg).getStats();

        if(log.isDebugEnabled()) log.debug("End getWorkerStats("+res+")");
        
        return res;
    }

    public GridStats getGridStats() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start getGridStats()");

        GridStats res;
        super.getGridMessageChannel().send(new GridMessageAdminGetGridStats());
        GridMessage msg=super.getGridMessageChannel().recv(30*1000);

        if((msg==null) || (!(msg instanceof GridMessageAdminGridStats)))
            res=null;
        else
            res=((GridMessageAdminGridStats)msg).getStats();

        if(log.isDebugEnabled()) log.debug("End getGridStats("+res+")");
        
        return res;
    }
}
