/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.client;

import java.io.*;
import java.net.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.server.*;
import org.homedns.dade.jcgrid.vfs.*;
import org.homedns.dade.jcgrid.util.*;

public class GridClient extends GridNodeGeneric {
    private final static String className=GridClient.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private vfsSession session;

    public GridClient() {
        super(new GridNodeClientConfig());

        if(log.isDebugEnabled()) log.debug("Start GridClient()");

        session=null;
        
        super.getNodeConfig().setWorkingDir(System.getProperty("user.dir"));

        if(log.isDebugEnabled()) log.debug("End GridClient()");
    }

    public void send(GridMessage msg) throws Exception {
        if(log.isDebugEnabled()) log.debug("Start send("+msg+")");

        super.getGridMessageChannel().send(msg);

        if(log.isDebugEnabled()) log.debug("End send()");
    }

    public GridMessage recv() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start recv()");

        GridMessage msg=super.getGridMessageChannel().recv();

        if(log.isDebugEnabled()) log.debug("End recv("+msg+")");
        
        return msg;
    }

    //----------------------------- Start & Stop -------------------------------

    public void start() throws Exception {
        if(log.isDebugEnabled()) logDetail.debug("Start start()");

        super.start();

        if(this.getNodeConfig().getGridConfig().getUseVFS()) {
            session=new vfsSession(super.getNodeConfig().getWorkingDir());
            session.start();
            log.warn("VFS session size: "+session.getSize()/(1024*1024)+"MB");

            session.syncVFSSession(super.getGridMessageChannel());
        }
        
        if(log.isDebugEnabled()) log.debug("End start()");
    }
}
