/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd;

import java.io.*;
import java.util.*;

import org.apache.log4j.*;
import org.apache.commons.cli.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.server.*;
import org.homedns.dade.jcgrid.worker.*;
import org.homedns.dade.jcgrid.client.*;
import org.homedns.dade.jcgrid.admin.*;

public class MainCmd {
    public MainCmd() { }

    public static void setUpLog4J(String name, boolean consoleOutput) throws Exception {
        String log4jCfgFileName=System.getProperty("user.dir")+
            File.separator+"log4j.cfg";
        File log4jCfgFile=new File(log4jCfgFileName);
        if(log4jCfgFile.exists())
            PropertyConfigurator.configureAndWatch(log4jCfgFileName,15*1000);
        else {
            // The cfg file doesn't exist so we are going to use the default
            // configuration

            BasicConfigurator.configure();

            Logger.getLogger("org.homedns.dade.jcgrid").setLevel(Level.WARN);
            Logger.getLogger("DETAIL.org.homedns.dade.jcgrid").setLevel(Level.WARN);

            Logger.getRootLogger().removeAllAppenders();
            Appender app=new RollingFileAppender(
                new PatternLayout("%-4r %d [%t] %-5p %x %m%n"),
                "jcgrid-"+name+".log",
                true);
            Logger.getRootLogger().addAppender(app);

            if(consoleOutput) {
                Appender capp=new ConsoleAppender(
                    new PatternLayout("%-4r %d [%t] %-5p %x %m%n"));
                Logger.getRootLogger().addAppender(capp);
            }
        }
    }
    
    public static CommandLine parseCommonOptions(Options options,
        GridNodeConfig nodeConfig, String[] args) throws Exception {
        // Parse common command line options

        options.addOption("s",true,"set server address");
        options.addOption("p",true,"set server client communication port");
        options.addOption("k",true,"set server woker communication port");
        options.addOption("a",true,"set server admin. communication port");
        options.addOption("w",true,"set server password");
        options.addOption("d",true,"set working directory");
        options.addOption("e",false,"enable secure connection");
        options.addOption("z",false,"enable compressed connection");
        
        if(GridNodeConfig.TYPE_WORKER.equals(nodeConfig.getType()))
            options.addOption("c",true,"set the number of workers to run");
        
        if((GridNodeConfig.TYPE_WORKER.equals(nodeConfig.getType())) ||
            (GridNodeConfig.TYPE_CLIENT.equals(nodeConfig.getType())) ||
            (GridNodeConfig.TYPE_ADMIN.equals(nodeConfig.getType())))
            options.addOption("n",true,"set server session name");
            

        CommandLineParser parser=new PosixParser();
        CommandLine cmd=parser.parse(options,args);

        GridConfig config=nodeConfig.getGridConfig();
        if(cmd.hasOption("s"))
            config.setServerAddress(cmd.getOptionValue("s"));
        if(cmd.hasOption("p"))
            config.setServerClientPort(Integer.parseInt(cmd.getOptionValue("p")));
        if(cmd.hasOption("k"))
            config.setServerWorkerPort(Integer.parseInt(cmd.getOptionValue("k")));
        if(cmd.hasOption("a"))
            config.setServerAdminPort(Integer.parseInt(cmd.getOptionValue("a")));
        if(cmd.hasOption("w"))
            config.setServerPassword(cmd.getOptionValue("w"));
        if(cmd.hasOption("e"))
            config.setUseSecureConnection(true);
        if(cmd.hasOption("z"))
            config.setUseCompression(true);

        if(cmd.hasOption("d"))
            nodeConfig.setWorkingDir(cmd.getOptionValue("d"));
        
        if(cmd.hasOption("?")) {
            HelpFormatter formatter=new HelpFormatter();
            formatter.printHelp("JCGrid command",options);

            System.exit(0);
        }

        if(GridNodeConfig.TYPE_WORKER.equals(nodeConfig.getType())) {
            GridNodeWorkerConfig wCfg=(GridNodeWorkerConfig)nodeConfig;

            if(cmd.hasOption("c"))
                wCfg.setWorkerCount(Integer.parseInt(cmd.getOptionValue("c")));
        }

        if((GridNodeConfig.TYPE_WORKER.equals(nodeConfig.getType())) ||
            (GridNodeConfig.TYPE_CLIENT.equals(nodeConfig.getType())) ||
            (GridNodeConfig.TYPE_ADMIN.equals(nodeConfig.getType()))) {
            GridNodeGenericConfig gCfg=(GridNodeGenericConfig)nodeConfig;

            if(cmd.hasOption("n"))
                gCfg.setSessionName(cmd.getOptionValue("n"));
        }
        
        return cmd;
    }
}
