/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.dummy;

import org.apache.log4j.*;
import org.apache.commons.cli.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.cmd.*;
import org.homedns.dade.jcgrid.client.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.worker.impl.dummy.*;

public class JCGridClient {
    private final static String className=JCGridClient.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private final static int WORKREQUEST_COUNT=100;
    
    public static void main(String[] args) {
        try {
            // Setup log4j

            MainCmd.setUpLog4J("client",true);

            // Setup GridServer

            log.warn("-----------------------------------------------");
            log.warn("-- JCGridClient Dummy v"+Version.RELEASE);
            log.warn("-----------------------------------------------");
            
            GridClient gc=new GridClient();

            // Parse command line options

            Options options=new Options();
            try {
                CommandLine cmd=MainCmd.parseCommonOptions(
                    options,gc.getNodeConfig(),args);

                if(cmd.getArgs().length>0)
                    throw new Exception("Unknown command line option");
            } catch(Exception ex) {
                log.warn("Error while parsing command line",ex);

                HelpFormatter formatter=new HelpFormatter();
                formatter.printHelp("JCGridClient",options);

                System.exit(0);
            }

            // Start Client

            gc.start();

            log.warn("Sending work requests...");

            boolean[]  resultReceived=new boolean[WORKREQUEST_COUNT];
            DummyWorkRequest[] dwr=new DummyWorkRequest[WORKREQUEST_COUNT];
            for(int i=0;i<WORKREQUEST_COUNT;i++) {
                resultReceived[i]=false;
                dwr[i]=new DummyWorkRequest(((GridNodeGenericConfig)gc.getNodeConfig())
                    .getSessionName(),i);
                gc.send(new GridMessageWorkRequest(dwr[i]));
            }

            log.warn("Receiving work results...");
            for(int i=0;i<WORKREQUEST_COUNT;i++) {
                GridMessageWorkResult gmwr=(GridMessageWorkResult)gc.recv();
                DummyWorkResult dwres=(DummyWorkResult)gmwr.getWorkResult();

                int id=dwres.getRID();
                resultReceived[id]=true;

                if(dwres.getNumber()!=dwr[id].getNumber())
                    log.warn("Message "+id+" match failed !");
            }
            
            boolean res=true;
            for(int i=0;i<WORKREQUEST_COUNT;i++)
                res=res && resultReceived[i];

            gc.stop();
            
            if(res)
                log.warn("Test completed.");
            else
                log.warn("Test failed.");
        } catch(Exception ex) {
            log.warn("Error",ex);
            System.exit(0);
        }
    }
}
