/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.mandel;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.cmd.*;
import org.homedns.dade.jcgrid.client.*;
import org.homedns.dade.jcgrid.worker.*;
import org.homedns.dade.jcgrid.worker.impl.mandel.*;
import org.homedns.dade.jcgrid.admin.*;
import org.homedns.dade.jcgrid.gui.*;

public class guiJCGridClient extends javax.swing.JFrame {
    private final static String className=guiJCGridClient.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private static final long serialVersionUID=1L;

    private guiMandelRenderingPannel pPic;
    private guiRenderingSetting pSet;
    private guiJCGridAdminStatus pServerStatus;
    private guiMandelWorkerPannel pWorker;
    private guiJCGridAbout pAbout;

    private GridNodeClientConfig clientConfig;
    private GridNodeWorkerConfig workerConfig;
    private GridNodeAdminConfig adminConfig;

    /** Creates new form guiJCGridClient */
    public guiJCGridClient() {
        pSet=null;
        pServerStatus=null;
        pAbout=null;

        initComponents();

        clientConfig=new GridNodeClientConfig();
        clientConfig.autoSessioName();
        adminConfig=new GridNodeAdminConfig();
        adminConfig.autoSessioName();
        workerConfig=new GridNodeWorkerConfig();
        workerConfig.autoSessioName();

        // Share the same config

        GridConfig gc=clientConfig.getGridConfig();
        gc.setServerAddress(MandelConfig.MANDELGRID_SERVER);
        gc.setServerClientPort(MandelConfig.MANDELGRID_CLIENT_PORT);
        gc.setServerWorkerPort(MandelConfig.MANDELGRID_WORKER_PORT);
        gc.setServerAdminPort(MandelConfig.MANDELGRID_ADMIN_PORT);
        gc.setUseCompression(MandelConfig.MANDELGRID_USE_COMPRESSION);
        gc.setUseVFS(false);

        adminConfig.setGridConfig(gc);
        workerConfig.setGridConfig(gc);

        // Rendering pannel
        
        pPic=new guiMandelRenderingPannel();
        GridBagConstraints gridBagConstraints=new java.awt.GridBagConstraints();
        gridBagConstraints.fill=java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx=1.0;
        gridBagConstraints.weighty=1.0;
        this.getContentPane().add(pPic,gridBagConstraints);
        
        // Rendering settings pannel
        
        pSet=new guiRenderingSetting(clientConfig,this,false,pPic);
        pSet.setVisible(true);

        // Worker pannel
        
        pWorker=new guiMandelWorkerPannel(workerConfig,this,false);
        pWorker.setVisible(true);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        mBar = new javax.swing.JMenuBar();
        mFile = new javax.swing.JMenu();
        mSaveImageAs = new javax.swing.JMenuItem();
        mSepa2 = new javax.swing.JSeparator();
        mAbout = new javax.swing.JMenuItem();
        mSepa3 = new javax.swing.JSeparator();
        mExit = new javax.swing.JMenuItem();
        mEdit = new javax.swing.JMenu();
        mResetView = new javax.swing.JMenuItem();
        mAdmin = new javax.swing.JMenu();
        mStatus = new javax.swing.JMenuItem();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("JCGrid Mandel Client");
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        mFile.setText("File");
        mSaveImageAs.setText("Save image as...");
        mSaveImageAs.setMnemonic('s');
        mSaveImageAs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mSaveImageAsActionPerformed(evt);
            }
        });

        mFile.add(mSaveImageAs);

        mFile.add(mSepa2);

        mAbout.setText("About");
        mAbout.setMnemonic('a');
        mAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mAboutActionPerformed(evt);
            }
        });

        mFile.add(mAbout);

        mFile.add(mSepa3);

        mExit.setText("Exit");
        mExit.setMnemonic('e');
        mExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mExitActionPerformed(evt);
            }
        });

        mFile.add(mExit);

        mBar.add(mFile);

        mEdit.setText("Edit");
        mResetView.setText("Reset view");
        mResetView.setMnemonic('r');
        mResetView.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mResetViewActionPerformed(evt);
            }
        });

        mEdit.add(mResetView);

        mBar.add(mEdit);

        mAdmin.setText("Admin.");
        mStatus.setText("Server status");
        mStatus.setMnemonic('s');
        mStatus.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mStatusActionPerformed(evt);
            }
        });

        mAdmin.add(mStatus);

        mBar.add(mAdmin);

        setJMenuBar(mBar);

        setBounds(0, 0, 400, 300);
    }//GEN-END:initComponents

    private void mAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mAboutActionPerformed
        if((pAbout==null) || (!pAbout.isVisible())) {
            pAbout=new guiJCGridAbout(this,false);
            pAbout.setVisible(true);
        }
    }//GEN-LAST:event_mAboutActionPerformed

    private void mResetViewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mResetViewActionPerformed
        if(pSet!=null)
             pSet.startRendering();
    }//GEN-LAST:event_mResetViewActionPerformed

    private void mStatusActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mStatusActionPerformed
        if((pServerStatus!=null) && (pServerStatus.isVisible()))
            pServerStatus.refreshInfo(adminConfig);
        else {
            try {
                pServerStatus=new guiJCGridAdminStatus(adminConfig,this,false);
                pServerStatus.setVisible(true);
            } catch(Exception ex) {
                log.warn("Error while opening the Admin. Status dialog",ex);

                JOptionPane.showMessageDialog(this,"Error while opening the Admin. Status dialog",
                    "Admin error",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_mStatusActionPerformed

    private void mExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mExitActionPerformed
        this.exitForm(null);
    }//GEN-LAST:event_mExitActionPerformed

    private void mSaveImageAsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mSaveImageAsActionPerformed
        JFileChooser chooser=new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));

        int res=chooser.showSaveDialog(this);

        if(res==JFileChooser.APPROVE_OPTION)
            pPic.saveImage(chooser.getSelectedFile().getAbsolutePath());
    }//GEN-LAST:event_mSaveImageAsActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        try {
            // Setup log4j

            MainCmd.setUpLog4J("gui-client",false);

            new guiJCGridClient().setVisible(true);
        } catch(Exception ex) {
            log.warn("Error",ex);
            System.exit(0);
        }
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem mAbout;
    private javax.swing.JMenu mAdmin;
    private javax.swing.JMenuBar mBar;
    private javax.swing.JMenu mEdit;
    private javax.swing.JMenuItem mExit;
    private javax.swing.JMenu mFile;
    private javax.swing.JMenuItem mResetView;
    private javax.swing.JMenuItem mSaveImageAs;
    private javax.swing.JSeparator mSepa2;
    private javax.swing.JSeparator mSepa3;
    private javax.swing.JMenuItem mStatus;
    // End of variables declaration//GEN-END:variables
    
}
