/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.mandel;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.cmd.*;
import org.homedns.dade.jcgrid.worker.*;
import org.homedns.dade.jcgrid.worker.impl.mandel.*;

public class guiMandelWorkerPannel extends javax.swing.JDialog {
    private final static String className=guiMandelWorkerPannel.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private static final long serialVersionUID=1L;

    //--------------------------------------------------------------------------

    private class WorkerFeedback implements GridWorkerFeedback {
        private Color[] colorTable;
        private String lastSessionName;

        public WorkerFeedback() {
        }

        public void setMaxIter(int maxIter) {
            colorTable=new Color[maxIter];

            float r;
            float g;
            float b;

            float dr;
            float dg;
            float db;

            r=1.0f;
            g=0.0f;
            b=0.0f;

            dr=0.11f;
            dg=0.001f;
            db=0.001f;

            for(int i=0;i<maxIter-1;i++) {
                colorTable[i]=new Color(r,g,b);

                r+=dr;
                g+=dg;
                b+=db;

                if(r>1.0)
                    r-=1.0;
                if(g>1.0)
                    g-=1.0;
                if(b>1.0)
                    b-=1.0;
            }

            colorTable[maxIter-1]=Color.BLACK;
        }

        public void beginWorkingFor(String sessionName, WorkRequest req) {
            lStatus.setText("working for "+sessionName);
            lastSessionName=sessionName;
            
            MandelWorkRequest r=(MandelWorkRequest)req;
            this.setMaxIter(r.getMaxIter());
        }

        public void endWorkingFor(WorkResult res) {
            lStatus.setText("done work for "+lastSessionName);

            Graphics graph=guiMandelWorkerPannel.this.pImage.getGraphics();
            
            if(graph!=null) {
                MandelWorkResult r=(MandelWorkResult)res;

                graph.setPaintMode();
                int w=guiMandelWorkerPannel.this.pImage.getWidth();
                int h=guiMandelWorkerPannel.this.pImage.getHeight();
                int fw=r.getIter().length;
                int fh=r.getIter()[0].length;
                for(int y=0;y<fh;y++) {
                    for(int x=0;x<fw;x++) {
                        graph.setColor(
                            colorTable[((r.getIter())[x][y]) % colorTable.length]);

                        graph.fillRect(w/2-fw/2+x,h/2-fh/2+(fh-y),1,1);
                    }
                }
            }
        }

        public void start() {
        }
        
        public void stop() {
        }
    }

    //--------------------------------------------------------------------------

    private GridWorker worker;

    /** Creates new form guiMandelWorkerPannel */
    public guiMandelWorkerPannel(GridNodeWorkerConfig workerConfig,
        java.awt.Frame parent, boolean modal) {
        super(parent, modal);

        initComponents();

        worker=new GridWorker();
        worker.setNodeConfig(workerConfig);
        worker.setWorker(new MandelWorker());
        worker.setWorkerFeedback(new WorkerFeedback());
        
        try {
            worker.start();
        } catch(Exception ex) {
            log.warn("Error starting the JCGrid worker",ex);

            JOptionPane.showMessageDialog(this,"Error starting the JCGrid worker.",
                "Worker error",JOptionPane.ERROR_MESSAGE);

            worker=null;
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pStatus = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        lStatus = new javax.swing.JLabel();
        pImage = new javax.swing.JPanel();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("JCGrid MandelWorker");
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        pStatus.setLayout(new java.awt.GridBagLayout());

        pStatus.setBorder(new javax.swing.border.TitledBorder("Worker status"));
        jLabel9.setText("Status:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pStatus.add(jLabel9, gridBagConstraints);

        lStatus.setText("...");
        lStatus.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pStatus.add(lStatus, gridBagConstraints);

        pImage.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.ipadx = 48;
        gridBagConstraints.ipady = 48;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pStatus.add(pImage, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pStatus, gridBagConstraints);

        setBounds(100, 30, 300, 200);
    }//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel lStatus;
    private javax.swing.JPanel pImage;
    private javax.swing.JPanel pStatus;
    // End of variables declaration//GEN-END:variables
    
}
