/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.mandel;

import java.io.*;
import java.text.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.client.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.worker.impl.mandel.*;
import org.homedns.dade.jcgrid.server.*;

public class guiRenderingSetting extends javax.swing.JDialog {
    private final static String className=guiRenderingSetting.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);
    
    private static final long serialVersionUID=1L;

    //--------------------------------------------------------------------------

    private class RenderingThreadFeedback implements MandelRenderingClientFeedback {
        public RenderingThreadFeedback() {
        }
        
        public void error(String msg, Exception ex) {
            log.warn(msg,ex);

            JOptionPane.showMessageDialog(guiRenderingSetting.this,msg,
                "Rendering error",JOptionPane.ERROR_MESSAGE);
        }
        
        public void receivedFragmentResult(MandelWorkRequest req,
            MandelWorkResult res,
            MandelRenderingFrame frame) {
            pPic.updateImage(frame);
        }
        
        public void renderingBegin() {
            pbStatus.setValue(pbStatus.getMinimum());
        }
        
        public void renderingEnd() {            
            rendThread=null;
            bStartStop.setText("Start rendering");
        }
        
        public void sendingFrgamentRequest(MandelWorkRequest req) {
        }
        
        public void setProgressMaximum(int max) {
            pbStatus.setMaximum(max);
        }
        
        public void setProgressMinimum(int min) {
            pbStatus.setMinimum(min);
        }
        
        public void setProgressValue(int val) {
            pbStatus.setValue(val);
        }
        
        public void setRenderingTime(MandelWorkRequest req, long msdt) {
            double dt=msdt/1000.0;
            double pixsec=req.getXStep()*req.getYStep()/dt;

            DecimalFormat df=new DecimalFormat("0.00");

            if(dt>0) 
                lRenderTime.setText(df.format(dt/60.0)+" min ("+
                    df.format(pixsec)+"pixel/sec)");
            else
                lRenderTime.setText("N/A");
        }        
        
        public void completeFrame(int idx, MandelRenderingFrame frame) {
            pPic.updateImage(frame);
        }
    }

    //--------------------------------------------------------------------------
    
    private GridNodeClientConfig clientConfig;

    private guiMandelRenderingPannel pPic;

    private short maxIteration;
    private double zoomFactor;
    private double xOrigin;
    private double yOrigin;
    private int imgWidth;
    private int imgHeight;
    
    private MandelRenderingClient rendThread;
    private MandelWorkRequest lastRequest;
    
    /** Creates new form guiRenderingSetting */
    public guiRenderingSetting(GridNodeClientConfig cfg,
        java.awt.Frame parent, boolean modal,
        guiMandelRenderingPannel rendPannel) {
        super(parent, modal);
        
        clientConfig=cfg;
        pPic=rendPannel;
        rendThread=null;
        lastRequest=null;

        initComponents();

        this.resetView(false);
    }

    public void resetView(boolean doRendering) {
        maxIteration=512;
        sMaxIteration.setValue(maxIteration);
        lMaxIteration.setText(Short.toString(maxIteration));
        zoomFactor=1.0;
        lZoomFactor.setText(Double.toString(zoomFactor));
        xOrigin=0.0;
        lXOrig.setText(Double.toString(xOrigin));
        yOrigin=0.0;
        lYOrig.setText(Double.toString(yOrigin));

        if(doRendering)
            this.startRendering();
    }
    
    public void startRendering() {
        pPic.initImage();
        
        int colorType=MandelRenderingFrame.COLOR_TABLE_TYPE_RED;
        if(cColors.getSelectedIndex()==MandelRenderingFrame.COLOR_TABLE_TYPE_MISC)
            colorType=MandelRenderingFrame.COLOR_TABLE_TYPE_MISC;

        short mi=maxIteration;
        double zf=zoomFactor;
        double xo=xOrigin;
        double yo=yOrigin;
        int w=pPic.getWidth();
        int h=pPic.getHeight();

        if((w<=0) || (h<=0))
            return;

        lImgWidth.setText(Integer.toString(w));
        lImgHeight.setText(Integer.toString(h));
        float ratio=h/(float)w;            

        // Check if there is a rendering already running

        if(rendThread!=null) {
            try {
                rendThread.interrupt();
                rendThread.join();
            } catch(Exception ex) {
                log.warn("Error stopping the rendering",ex);

                JOptionPane.showMessageDialog(this,"Error stopping the rendering.",
                    "Rendering error",JOptionPane.ERROR_MESSAGE);
                
                rendThread=null;
            }
        }
        
        // Setup JCGrid Client

        GridClient gc=new GridClient();
        gc.setNodeConfig(clientConfig);
        GridNodeClientConfig config=(GridNodeClientConfig)gc.getNodeConfig();

        try {
            // Creating work request

            lastRequest=new MandelWorkRequest(
                config.getSessionName(),0,
                (short)mi,
                xo-2.0*zf,yo-2.0*zf*ratio,
                4.0*zf/w,4.0*zf*ratio/h,w,h);

            rendThread=new MandelRenderingClient(
                config,new RenderingThreadFeedback(),lastRequest);
            rendThread.getFrame().setColorTableType(colorType);
            rendThread.getFrame().setMaxIter(mi);

            rendThread.start();
        } catch(Exception ex) {
            log.warn("Error starting the rendering",ex);

            JOptionPane.showMessageDialog(this,"Error starting the rendering.",
                "Rendering error",JOptionPane.ERROR_MESSAGE);

            rendThread=null;
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pCfg = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        lMaxIteration = new javax.swing.JLabel();
        sMaxIteration = new javax.swing.JSlider();
        jLabel5 = new javax.swing.JLabel();
        lZoomFactor = new javax.swing.JLabel();
        bZoomIn = new javax.swing.JButton();
        bZoomOut = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        lXOrig = new javax.swing.JLabel();
        lLeft = new javax.swing.JButton();
        lRight = new javax.swing.JButton();
        jLabel7 = new javax.swing.JLabel();
        lYOrig = new javax.swing.JLabel();
        lUp = new javax.swing.JButton();
        lDown = new javax.swing.JButton();
        bCursor = new javax.swing.JButton();
        jLabel9 = new javax.swing.JLabel();
        lImgWidth = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        lImgHeight = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        cColors = new javax.swing.JComboBox();
        pRendering = new javax.swing.JPanel();
        bStartStop = new javax.swing.JButton();
        pbStatus = new javax.swing.JProgressBar();
        jLabel13 = new javax.swing.JLabel();
        lRenderTime = new javax.swing.JLabel();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("Rendering settings");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setName("dSetting");
        pCfg.setLayout(new java.awt.GridBagLayout());

        pCfg.setBorder(new javax.swing.border.TitledBorder("Configuration"));
        jLabel4.setText("Max. iterations:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel4, gridBagConstraints);

        lMaxIteration.setText("...");
        lMaxIteration.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lMaxIteration, gridBagConstraints);

        sMaxIteration.setMinimum(512);
        sMaxIteration.setMaximum(30000);
        sMaxIteration.setSnapToTicks(true);
        sMaxIteration.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sMaxIterationStateChanged(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(sMaxIteration, gridBagConstraints);

        jLabel5.setText("Zoom factor:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel5, gridBagConstraints);

        lZoomFactor.setText("...");
        lZoomFactor.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lZoomFactor, gridBagConstraints);

        bZoomIn.setText("Zoom In");
        bZoomIn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bZoomInActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(bZoomIn, gridBagConstraints);

        bZoomOut.setText("Zoom Out");
        bZoomOut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bZoomOutActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(bZoomOut, gridBagConstraints);

        jLabel6.setText("X origin:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel6, gridBagConstraints);

        lXOrig.setText("...");
        lXOrig.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lXOrig, gridBagConstraints);

        lLeft.setText("Move left");
        lLeft.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lLeftActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lLeft, gridBagConstraints);

        lRight.setText("Move right");
        lRight.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lRightActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lRight, gridBagConstraints);

        jLabel7.setText("Y origin:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel7, gridBagConstraints);

        lYOrig.setText("...");
        lYOrig.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lYOrig, gridBagConstraints);

        lUp.setText("Move up");
        lUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lUpActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lUp, gridBagConstraints);

        lDown.setText("Move down");
        lDown.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lDownActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lDown, gridBagConstraints);

        bCursor.setText("Move to cursor");
        bCursor.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCursorActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(bCursor, gridBagConstraints);

        jLabel9.setText("Image width:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel9, gridBagConstraints);

        lImgWidth.setText("...");
        lImgWidth.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lImgWidth, gridBagConstraints);

        jLabel8.setText("Image height:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel8, gridBagConstraints);

        lImgHeight.setText("...");
        lImgHeight.setForeground(java.awt.Color.blue);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 11;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(lImgHeight, gridBagConstraints);

        jLabel10.setText("Image colors:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(jLabel10, gridBagConstraints);

        cColors.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Red", "Misc" }));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 12;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pCfg.add(cColors, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pCfg, gridBagConstraints);

        pRendering.setLayout(new java.awt.GridBagLayout());

        pRendering.setBorder(new javax.swing.border.TitledBorder("Rendering"));
        bStartStop.setText("Start rendering");
        bStartStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bStartStopActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(bStartStop, gridBagConstraints);

        pbStatus.setStringPainted(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(pbStatus, gridBagConstraints);

        jLabel13.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel13.setText("Last rendering time:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(jLabel13, gridBagConstraints);

        lRenderTime.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lRenderTime.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(lRenderTime, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pRendering, gridBagConstraints);

        setBounds(50, 50, 600, 320);
    }//GEN-END:initComponents

    private void bCursorActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bCursorActionPerformed
        if(lastRequest!=null) {
            xOrigin=lastRequest.getXOrig()+pPic.getCursorX()*
                lastRequest.getDX()*lastRequest.getXStep();
            lXOrig.setText(Double.toString(xOrigin));

            yOrigin=lastRequest.getYOrig()+pPic.getCursorY()*
                lastRequest.getDY()*lastRequest.getYStep();
            lYOrig.setText(Double.toString(yOrigin));
        }
    }//GEN-LAST:event_bCursorActionPerformed

    private void lDownActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lDownActionPerformed
        yOrigin-=0.1*zoomFactor;
        lYOrig.setText(Double.toString(yOrigin));
    }//GEN-LAST:event_lDownActionPerformed

    private void lUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lUpActionPerformed
        yOrigin+=0.1*zoomFactor;
        lYOrig.setText(Double.toString(yOrigin));
    }//GEN-LAST:event_lUpActionPerformed

    private void lRightActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lRightActionPerformed
        xOrigin+=0.1*zoomFactor;
        lXOrig.setText(Double.toString(xOrigin));
    }//GEN-LAST:event_lRightActionPerformed

    private void lLeftActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lLeftActionPerformed
        xOrigin-=0.1*zoomFactor;
        lXOrig.setText(Double.toString(xOrigin));
    }//GEN-LAST:event_lLeftActionPerformed

    private void bZoomOutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bZoomOutActionPerformed
        zoomFactor*=1.0/0.925;
        lZoomFactor.setText(Double.toString(zoomFactor));
    }//GEN-LAST:event_bZoomOutActionPerformed

    private void bZoomInActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bZoomInActionPerformed
        zoomFactor*=0.925;
        lZoomFactor.setText(Double.toString(zoomFactor));
    }//GEN-LAST:event_bZoomInActionPerformed

    private void sMaxIterationStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sMaxIterationStateChanged
        maxIteration=(short)sMaxIteration.getValue();
        lMaxIteration.setText(Short.toString(maxIteration));
    }//GEN-LAST:event_sMaxIterationStateChanged

    private void bStartStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bStartStopActionPerformed
        if(rendThread==null)
            this.startRendering();
        else {
            try {
                rendThread.interrupt();
                rendThread.join();
            } catch(Exception ex) {
                log.warn("Error stopping the rendering",ex);

                JOptionPane.showMessageDialog(this,"Error stopping the rendering.",
                    "Rendering error",JOptionPane.ERROR_MESSAGE);
                
                rendThread=null;
            }
        }
        
        if(rendThread==null)
            bStartStop.setText("Start rendering");
        else
            bStartStop.setText("Stop rendering");
    }//GEN-LAST:event_bStartStopActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bCursor;
    private javax.swing.JButton bStartStop;
    private javax.swing.JButton bZoomIn;
    private javax.swing.JButton bZoomOut;
    private javax.swing.JComboBox cColors;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JButton lDown;
    private javax.swing.JLabel lImgHeight;
    private javax.swing.JLabel lImgWidth;
    private javax.swing.JButton lLeft;
    private javax.swing.JLabel lMaxIteration;
    private javax.swing.JLabel lRenderTime;
    private javax.swing.JButton lRight;
    private javax.swing.JButton lUp;
    private javax.swing.JLabel lXOrig;
    private javax.swing.JLabel lYOrig;
    private javax.swing.JLabel lZoomFactor;
    private javax.swing.JPanel pCfg;
    private javax.swing.JPanel pRendering;
    private javax.swing.JProgressBar pbStatus;
    private javax.swing.JSlider sMaxIteration;
    // End of variables declaration//GEN-END:variables
    
}
