/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.povray;

import org.apache.log4j.*;
import org.apache.commons.cli.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.cmd.*;
import org.homedns.dade.jcgrid.worker.*;
import org.homedns.dade.jcgrid.worker.impl.povray.*;

public class JCGridWorker {
    private final static String className=JCGridWorker.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    public static void main(String[] args) {
        try {
            // Setup log4j

            MainCmd.setUpLog4J("worker",true);

            // Setup GridServer

            log.warn("-----------------------------------------------");
            log.warn("-- JCGridWorker POVRay v"+Version.RELEASE);
            log.warn("-----------------------------------------------");

            GridNodeWorkerConfig config=new GridNodeWorkerConfig();

            // Try to guess if we are running under Windows

            String osName=System.getProperty("os.name" );
            log.warn("OS Name: "+osName);

            boolean usePVEngineEXE;
            if(osName.matches("Windows.*"))
                usePVEngineEXE=true;
            else
                usePVEngineEXE=false;

            // Parse command line options

            Options options=new Options();
            options.addOption("v",false,"Execute pvengine.exe instead of povray command");
            try {
                CommandLine cmd=MainCmd.parseCommonOptions(options,config,args);
     
                // POVRay options

                if(cmd.hasOption("v"))
                    usePVEngineEXE=true;

                if(cmd.getArgs().length>0)
                    throw new Exception("Unknown command line option");
            } catch(Exception ex) {
                log.warn("Error while parsing command line",ex);

                HelpFormatter formatter=new HelpFormatter();
                formatter.printHelp("JCGridWorker",options);

                System.exit(0);
            }

            // Start all required Workers

            GridWorker[] gw=new GridWorker[config.getWorkerCount()];
            for(int i=0;i<config.getWorkerCount();i++) {
                gw[i]=new GridWorker();
                gw[i].setNodeConfig((GridNodeGenericConfig)config.clone());

                ((GridNodeGenericConfig)gw[i].getNodeConfig()).
                    setSessionName(config.getSessionName()+"_"+i);
                ((GridNodeGenericConfig)gw[i].getNodeConfig()).
                    setWorkingDir(config.getWorkingDir()+"_"+i);
                gw[i].setWorker(new POVWorker(usePVEngineEXE));

                gw[i].start();
                log.warn("Running worker "+i+"...");
            }
            
            // Wait shutdown

            for(int i=0;i<config.getWorkerCount();i++)
                gw[i].waitShutdown();
        } catch(Exception ex) {
            log.warn("Error",ex);
            System.exit(0);
        }
    }
}
