/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.cmd.povray;

import java.io.*;
import java.text.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.client.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.worker.impl.povray.*;
import org.homedns.dade.jcgrid.server.*;

public class guiRenderingSetting extends javax.swing.JDialog {
    private final static String className=guiRenderingSetting.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    //--------------------------------------------------------------------------

    private class RenderingThreadFeedback implements POVRenderingClientFeedback {
        public RenderingThreadFeedback() {
        }
        
        public void error(String msg, Exception ex) {
            log.warn(msg,ex);

            JOptionPane.showMessageDialog(guiRenderingSetting.this,msg,
                "Rendering error",JOptionPane.ERROR_MESSAGE);
        }
        
        public void receivedFragmentResult(POVWorkRequest req, POVWorkResult res,
            int idx, POVRenderingFrame frame) {
            pPic.updateImage(frame);
        }
        
        public void renderingBegin() {
            pbStatus.setValue(pbStatus.getMinimum());
        }
        
        public void renderingEnd() {            
            rendThread=null;
            bStartStop.setText("Start rendering");
        }
        
        public void sendingFrgamentRequest(POVWorkRequest req) {
        }
        
        public void setProgressMaximum(int max) {
            pbStatus.setMaximum(max);
        }
        
        public void setProgressMinimum(int min) {
            pbStatus.setMinimum(min);
        }
        
        public void setProgressValue(int val) {
            pbStatus.setValue(val);
        }
        
        public void setRenderingTime(POVWorkRequest req, long msdt) {
            double dt=msdt/1000.0;
            double pixsec=req.getWidth()*req.getHeight()/dt;

            DecimalFormat df=new DecimalFormat("0.00");

            if(dt>0) 
                lRenderTime.setText(df.format(dt/60.0)+" min ("+
                    df.format(pixsec)+"pixel/sec)");
            else
                lRenderTime.setText("N/A");
        }        
        
        public void completeFrame(int idx, POVRenderingFrame frame) {
            pPic.updateImage(frame);
        }
    }

    //--------------------------------------------------------------------------
    
    private static final long serialVersionUID=1L;

    private GridNodeClientConfig clientConfig;

    private guiPOVRenderingPannel pPic;
    private POVRenderingClient rendThread;

    /** Creates new form guiRenderingSetting */
    public guiRenderingSetting(GridNodeClientConfig cfg,
        java.awt.Frame parent, boolean modal,
        guiPOVRenderingPannel rendPannel) {
        super(parent, modal);
        
        clientConfig=cfg;
        pPic=rendPannel;
        rendThread=null;

        initComponents();
    }

    public void saveSettings(String fileName) {
        try {
            Properties prop=clientConfig.toProperties();

            prop.put("povray.scene.sessionname",tSessionName.getText().trim());
            prop.put("povray.scene.workingdirectory",tWorkDir.getText().trim());
            prop.put("povray.scene.inifile",tINIFile.getText().trim());
            
            prop.put("povray.image.width",tWidth.getText().trim());
            prop.put("povray.image.height",tHeight.getText().trim());
            prop.put("povray.image.fragment",tFragmentSize.getText().trim());
            prop.put("povray.image.startcolumn",tStartX.getText().trim());
            prop.put("povray.image.endcolumn",tEndX.getText().trim());
            prop.put("povray.image.startrow",tStartY.getText().trim());
            prop.put("povray.image.endrow",tEndY.getText().trim());
            
            prop.store(new FileOutputStream(new File(fileName)),
                "JCGrid GUI Client Settings");
        } catch(Exception ex) {
            log.warn("Error while saving settings.",ex);

            JOptionPane.showMessageDialog(this,"Error while saving settings.",
                "IO error",JOptionPane.ERROR_MESSAGE);
        }
    }

    public GridNodeClientConfig loadSettings(String fileName) {
        try {
            Properties prop=new Properties();
            prop.load(new FileInputStream(new File(fileName)));
            clientConfig=new GridNodeClientConfig(prop);

            tSessionName.setText(prop.getProperty("povray.scene.sessionname",""));
            tWorkDir.setText(prop.getProperty("povray.scene.workingdirectory",""));
            tINIFile.setText(prop.getProperty("povray.scene.inifile",""));

            tWidth.setText(prop.getProperty("povray.image.width",""));
            tHeight.setText(prop.getProperty("povray.image.height",""));
            tFragmentSize.setText(prop.getProperty("povray.image.fragment",""));
            tStartX.setText(prop.getProperty("povray.image.startcolumn",""));
            tEndX.setText(prop.getProperty("povray.image.endcolumn",""));
            tStartY.setText(prop.getProperty("povray.image.startrow",""));
            tEndY.setText(prop.getProperty("povray.image.endrow",""));
        } catch(Exception ex) {
            log.warn("Error while loading settings.",ex);

            JOptionPane.showMessageDialog(this,"Error while loading settings.",
                "IO error",JOptionPane.ERROR_MESSAGE);
            
            return null;
        }
        
        return clientConfig;
    }

    public String getSessionName() {
        String sessionName=tSessionName.getText().trim();
        if(!sessionName.matches("[a-zA-Z0-9_]+")) {
            JOptionPane.showMessageDialog(this,"Syntax error in the session name.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return null;
        }
        
        return sessionName;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pScene = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        tSessionName = new javax.swing.JTextField();
        bAutoSessionName = new javax.swing.JButton();
        jLabel8 = new javax.swing.JLabel();
        tWorkDir = new javax.swing.JTextField();
        bWorkDir = new javax.swing.JButton();
        jLabel12 = new javax.swing.JLabel();
        tINIFile = new javax.swing.JTextField();
        bINIFile = new javax.swing.JButton();
        pImage = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        tWidth = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        tHeight = new javax.swing.JTextField();
        bDefaultImageSize = new javax.swing.JButton();
        jLabel3 = new javax.swing.JLabel();
        tFragmentSize = new javax.swing.JTextField();
        bDefaultFragSize = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        tStartX = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        tEndX = new javax.swing.JTextField();
        jLabel6 = new javax.swing.JLabel();
        tStartY = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        tEndY = new javax.swing.JTextField();
        bDefaultStartEnd = new javax.swing.JButton();
        bClearImage = new javax.swing.JButton();
        pRendering = new javax.swing.JPanel();
        bStartStop = new javax.swing.JButton();
        pbStatus = new javax.swing.JProgressBar();
        jLabel13 = new javax.swing.JLabel();
        lRenderTime = new javax.swing.JLabel();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("Rendering settings");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setName("dSetting");
        pScene.setLayout(new java.awt.GridBagLayout());

        pScene.setBorder(new javax.swing.border.TitledBorder("Scene"));
        jLabel9.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel9.setText("Session name:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(jLabel9, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(tSessionName, gridBagConstraints);

        bAutoSessionName.setText("Auto");
        bAutoSessionName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bAutoSessionNameActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(bAutoSessionName, gridBagConstraints);

        jLabel8.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel8.setText("Working directory:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(jLabel8, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(tWorkDir, gridBagConstraints);

        bWorkDir.setText("Select");
        bWorkDir.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bWorkDirActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(bWorkDir, gridBagConstraints);

        jLabel12.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel12.setText("INI file:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(jLabel12, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(tINIFile, gridBagConstraints);

        bINIFile.setText("Default");
        bINIFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bINIFileActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pScene.add(bINIFile, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pScene, gridBagConstraints);

        pImage.setLayout(new java.awt.GridBagLayout());

        pImage.setBorder(new javax.swing.border.TitledBorder("Image"));
        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel1.setText("Width:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tWidth, gridBagConstraints);

        jLabel2.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel2.setText("Height:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel2, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tHeight, gridBagConstraints);

        bDefaultImageSize.setText("Default");
        bDefaultImageSize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultImageSizeActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(bDefaultImageSize, gridBagConstraints);

        jLabel3.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel3.setText("Fragment size:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel3, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tFragmentSize, gridBagConstraints);

        bDefaultFragSize.setText("Default");
        bDefaultFragSize.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultFragSizeActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(bDefaultFragSize, gridBagConstraints);

        jLabel4.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel4.setText("Start column:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel4, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tStartX, gridBagConstraints);

        jLabel5.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel5.setText("End column:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel5, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tEndX, gridBagConstraints);

        jLabel6.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel6.setText("Start row:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel6, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tStartY, gridBagConstraints);

        jLabel7.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel7.setText("End row:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(jLabel7, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(tEndY, gridBagConstraints);

        bDefaultStartEnd.setText("Default");
        bDefaultStartEnd.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultStartEndActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridheight = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(bDefaultStartEnd, gridBagConstraints);

        bClearImage.setText("Clear image");
        bClearImage.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bClearImageActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 7;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pImage.add(bClearImage, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.1;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pImage, gridBagConstraints);

        pRendering.setLayout(new java.awt.GridBagLayout());

        pRendering.setBorder(new javax.swing.border.TitledBorder("Rendering"));
        bStartStop.setText("Start rendering");
        bStartStop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bStartStopActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(bStartStop, gridBagConstraints);

        pbStatus.setStringPainted(true);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(pbStatus, gridBagConstraints);

        jLabel13.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel13.setText("Last rendering time:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(jLabel13, gridBagConstraints);

        lRenderTime.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lRenderTime.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pRendering.add(lRenderTime, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pRendering, gridBagConstraints);

        setBounds(50, 50, 400, 500);
    }//GEN-END:initComponents

    private void bAutoSessionNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bAutoSessionNameActionPerformed
        clientConfig.autoSessioName();
        tSessionName.setText(clientConfig.getSessionName());
    }//GEN-LAST:event_bAutoSessionNameActionPerformed

    private void bDefaultFragSizeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultFragSizeActionPerformed
        tFragmentSize.setText("64");
    }//GEN-LAST:event_bDefaultFragSizeActionPerformed

    private void bClearImageActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bClearImageActionPerformed
        pPic.clearImage();
    }//GEN-LAST:event_bClearImageActionPerformed

    private void bDefaultImageSizeActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultImageSizeActionPerformed
        tWidth.setText(Integer.toString(pPic.getWidth()));
        tHeight.setText(Integer.toString(pPic.getHeight()));
    }//GEN-LAST:event_bDefaultImageSizeActionPerformed

    private void bINIFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bINIFileActionPerformed
        tINIFile.setText("jcgrid.ini");
    }//GEN-LAST:event_bINIFileActionPerformed

    private void bDefaultStartEndActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultStartEndActionPerformed
        int width=-1;
        try {
            width=Integer.parseInt(tWidth.getText());
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(this,"Syntax error in the image width.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if((width<16) || (width>2048)) {
            JOptionPane.showMessageDialog(this,"The image width must be between 16 and 2048.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        int height=-1;
        try {
            height=Integer.parseInt(tHeight.getText());
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(this,"Syntax error in the image height.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if((height<16) || (height>2048)) {
            JOptionPane.showMessageDialog(this,"The image height must be between 16 and 2048.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        tStartX.setText("1");
        tStartY.setText("1");
        tEndX.setText(Integer.toString(width));
        tEndY.setText(Integer.toString(height));
    }//GEN-LAST:event_bDefaultStartEndActionPerformed

    private void bStartStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bStartStopActionPerformed
        if(rendThread==null) {
            // Check rendering settings

            // Scene

            String sessionName=this.getSessionName();
            if(sessionName==null)
                return;

            String workDir=tWorkDir.getText().trim();
            File fWorkDir=new File(workDir);
            if(!fWorkDir.exists()) {
                JOptionPane.showMessageDialog(this,"The working directory doesn't exists.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            } else if(!fWorkDir.isDirectory()) {
                JOptionPane.showMessageDialog(this,"The working directory isn't a directory.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            String iniFile=tINIFile.getText();
            try {
                File inif=new File(workDir+File.separator+iniFile);

                if(!inif.exists() || !inif.isFile()) {
                    JOptionPane.showMessageDialog(this,"The INI file doesn't exists or isn't a normal file.",
                        "Settings error",JOptionPane.ERROR_MESSAGE);
                    return;
                }
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"The INI file must be in the working directory.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            // Image

            int width=-1;
            try {
                width=Integer.parseInt(tWidth.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the image width.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((width<16) || (width>2048)) {
                JOptionPane.showMessageDialog(this,"The image width must be between 16 and 2048.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            int height=-1;
            try {
                height=Integer.parseInt(tHeight.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the image height.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((height<16) || (height>2048)) {
                JOptionPane.showMessageDialog(this,"The image height must be between 16 and 2048.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            int fragSize=-1;
            try {
                fragSize=Integer.parseInt(tFragmentSize.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the fragment size.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((fragSize<8) || (fragSize>width)) {
                JOptionPane.showMessageDialog(this,"The image fragment size must be between 8 and "+width+".",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            // StartX & EndX

            if(tStartX.getText().trim().equals(""))
                tStartX.setText("1");
            int startX=-1;
            try {
                startX=Integer.parseInt(tStartX.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the start column.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((startX<1) || (startX>width)) {
                JOptionPane.showMessageDialog(this,"The image start column must be between 1 and "+width+".",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            if(tEndX.getText().trim().equals(""))
                tEndX.setText(Integer.toString(width));
            int endX=-1;
            try {
                endX=Integer.parseInt(tEndX.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the end column.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((endX<startX+8) || (endX>width)) {
                JOptionPane.showMessageDialog(this,"The image end column must be between "+(startX+8)+" and "+width+".",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            // StartY & EndY

            if(tStartY.getText().trim().equals(""))
                tStartY.setText("1");
            int startY=-1;
            try {
                startY=Integer.parseInt(tStartY.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the start row.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((startY<1) || (startY>height)) {
                JOptionPane.showMessageDialog(this,"The image start row must be between 1 and "+height+".",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            if(tEndY.getText().trim().equals(""))
                tEndY.setText(Integer.toString(height));
            int endY=-1;
            try {
                endY=Integer.parseInt(tEndY.getText());
            } catch(Exception ex) {
                JOptionPane.showMessageDialog(this,"Syntax error in the end row.",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if((endX<startY+8) || (endY>height)) {
                JOptionPane.showMessageDialog(this,"The image end row must be between "+(startY+8)+" and "+height+".",
                    "Settings error",JOptionPane.ERROR_MESSAGE);
                return;
            }

            pPic.resizeImage(width,height);
            
            // Setup JCGrid Client

            GridClient gc=new GridClient();
            gc.setNodeConfig(clientConfig);
            GridNodeClientConfig config=(GridNodeClientConfig)gc.getNodeConfig();

            try {
                config.setSessionName(sessionName);
                config.setWorkingDir(workDir);

                // Creating work request

                POVWorkRequest povwr=new POVWorkRequest(config.getSessionName(),0,
                    iniFile,width,height,startX-1,endX-1,startY-1,endY-1);

                rendThread=new POVRenderingClient(
                    config,new RenderingThreadFeedback(),povwr,fragSize);
                rendThread.start();
            } catch(Exception ex) {
                log.warn("Error starting the rendering",ex);

                JOptionPane.showMessageDialog(this,"Error starting the rendering.",
                    "Rendering error",JOptionPane.ERROR_MESSAGE);
                
                rendThread=null;
            }
        } else {
            try {
                rendThread.interrupt();
                rendThread.join();
            } catch(Exception ex) {
                log.warn("Error stopping the rendering",ex);

                JOptionPane.showMessageDialog(this,"Error stopping the rendering.",
                    "Rendering error",JOptionPane.ERROR_MESSAGE);
                
                rendThread=null;
            }
        }
        
        if(rendThread==null)
            bStartStop.setText("Start rendering");
        else
            bStartStop.setText("Stop rendering");
    }//GEN-LAST:event_bStartStopActionPerformed

    private void bWorkDirActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bWorkDirActionPerformed
        JFileChooser chooser=new JFileChooser();
        chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));

        int res=chooser.showOpenDialog(this);

        if(res==JFileChooser.APPROVE_OPTION)
            tWorkDir.setText(chooser.getSelectedFile().getAbsolutePath());
    }//GEN-LAST:event_bWorkDirActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bAutoSessionName;
    private javax.swing.JButton bClearImage;
    private javax.swing.JButton bDefaultFragSize;
    private javax.swing.JButton bDefaultImageSize;
    private javax.swing.JButton bDefaultStartEnd;
    private javax.swing.JButton bINIFile;
    private javax.swing.JButton bStartStop;
    private javax.swing.JButton bWorkDir;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel lRenderTime;
    private javax.swing.JPanel pImage;
    private javax.swing.JPanel pRendering;
    private javax.swing.JPanel pScene;
    private javax.swing.JProgressBar pbStatus;
    private javax.swing.JTextField tEndX;
    private javax.swing.JTextField tEndY;
    private javax.swing.JTextField tFragmentSize;
    private javax.swing.JTextField tHeight;
    private javax.swing.JTextField tINIFile;
    private javax.swing.JTextField tSessionName;
    private javax.swing.JTextField tStartX;
    private javax.swing.JTextField tStartY;
    private javax.swing.JTextField tWidth;
    private javax.swing.JTextField tWorkDir;
    // End of variables declaration//GEN-END:variables
    
}
