/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.gui;


import java.io.*;
import java.text.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.admin.*;
import org.homedns.dade.jcgrid.message.*;

public class guiJCGridAdminStatus extends javax.swing.JDialog {
    private final static String className=guiJCGridAdminStatus.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private static final long serialVersionUID=1L;

    //------------------------- The JTable model -------------------------------
    
    private class WorkerStatsModel extends AbstractTableModel {
        private static final long serialVersionUID=1L;

        private String[] columnNames={
            "Node name", "Status", "Working for", "Work started", "Units done", "Unit/sec"
        };
        private Object[][] data;

        public WorkerStatsModel(Vector stats) {
            data=new Object[stats.size()][columnNames.length];

            int j=0;
            for(Iterator i=stats.iterator();i.hasNext();) {
                WorkerStats ws=(WorkerStats)i.next();
                
                data[j][0]=ws.getName();
                if(ws.getStatus()==WorkerStats.STATUS_BUSY) {
                    data[j][1]="Busy";
                    data[j][2]=ws.getWorkingFor();
                    SimpleDateFormat sdf=new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
                    data[j][3]=sdf.format(new Date(ws.getWorkingStarted()));
                } else {
                    data[j][1]="Free";
                    data[j][2]="";
                    data[j][3]="";
                }
                data[j][4]=Long.toString(ws.getUnitDone());
                data[j][5]=Double.toString(ws.getUnitSec());
                j++;
            }
        }

        public int getColumnCount() {
            return columnNames.length;
        }
        
        public int getRowCount() {
            return data.length;
        }
        
        public String getColumnName(int col) {
            return columnNames[col];
        }
        
        public Object getValueAt(int row, int col) {
            return data[row][col];
        }
        
        public Class getColumnClass(int c) {
            if(data.length>0)
                return this.getValueAt(0,c).getClass();
            else
                return Object.class;
        }
    }

    //--------------------------------------------------------------------------

    private JTable tStats;
    private GridNodeAdminConfig adminConfig;

    public guiJCGridAdminStatus(GridNodeAdminConfig cfg, java.awt.Frame parent, boolean modal) {
        super(parent,modal);

        initComponents();

        tStats=null;

        this.refreshInfo(cfg);
    }

    public void refreshInfo(GridNodeAdminConfig cfg) {
        try {
            adminConfig=cfg;

            lHostName.setText(adminConfig.getGridConfig().getServerAddress());
            lHostPort.setText(Integer.toString(
                adminConfig.getGridConfig().getServerAdminPort()));
            lSessionName.setText(adminConfig.getSessionName());

            GridAdmin ga=new GridAdmin();
            ga.setNodeConfig(adminConfig);

            ga.start();

            // Ping time
            
            long pingTime=ga.ping();
            lPing.setText(pingTime+"ms");
            
            // Grid Stats
            
            GridStats gs=ga.getGridStats();
            lWorkerCount.setText(Integer.toString(gs.getWorkerCount()));
            lQueueSize.setText(Integer.toString(gs.getWorkRequestsQueueSize()));
            
            // Worker stats
            
            Vector stats=ga.getWorkerStats();

            if(tStats!=null)
                sPane.remove(tStats);
            
            if(stats!=null) {
                WorkerStatsModel model=new WorkerStatsModel(stats);
                tStats=new javax.swing.JTable();
                tStats.setModel(model);
                sPane.setViewportView(tStats);

                TableColumn column=null;
                Component comp=null;
                int headerWidth=0;
                int cellWidth=0;
                TableCellRenderer headerRenderer=
                    tStats.getTableHeader().getDefaultRenderer();

                for(int i=0;i<model.getColumnCount();i++) {
                    column=tStats.getColumnModel().getColumn(i);

                    comp=headerRenderer.getTableCellRendererComponent(
                        null,column.getHeaderValue(),
                        false,false,0,0);
                    headerWidth=comp.getPreferredSize().width;

                    if(model.getRowCount()>0) {
                        comp=tStats.getDefaultRenderer(model.getColumnClass(i)).
                            getTableCellRendererComponent(
                                tStats,model.getValueAt(0,i),
                                false,false,0,i);
                        cellWidth=comp.getPreferredSize().width;
                    } else
                        cellWidth=0;

                    column.setPreferredWidth(Math.max(headerWidth,cellWidth));
                }
            }
            
            ga.stop();
            
            this.repaint();
        } catch(Exception ex) {
            log.warn("Error refreshing server information",ex);

            JOptionPane.showMessageDialog(this,"Error refreshing server information.",
                "Admin error",JOptionPane.ERROR_MESSAGE);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pServer = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        lHostName = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        lHostPort = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        lSessionName = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        lPing = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        lWorkerCount = new javax.swing.JLabel();
        jLabel15 = new javax.swing.JLabel();
        lQueueSize = new javax.swing.JLabel();
        pWorkers = new javax.swing.JPanel();
        sPane = new javax.swing.JScrollPane();
        bRefresh = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("JCGrid Server Status");
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        pServer.setLayout(new java.awt.GridBagLayout());

        pServer.setBorder(new javax.swing.border.TitledBorder("JCGrid Server"));
        jLabel10.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel10.setText("Server host name:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel10, gridBagConstraints);

        lHostName.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lHostName.setText("...");
        lHostName.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lHostName, gridBagConstraints);

        jLabel11.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel11.setText("Admin. server port:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel11, gridBagConstraints);

        lHostPort.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lHostPort.setText("...");
        lHostPort.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lHostPort, gridBagConstraints);

        jLabel13.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel13.setText("Admin. session name:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel13, gridBagConstraints);

        lSessionName.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lSessionName.setText("...");
        lSessionName.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lSessionName, gridBagConstraints);

        jLabel12.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel12.setText("Ping time:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel12, gridBagConstraints);

        lPing.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lPing.setText("...");
        lPing.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lPing, gridBagConstraints);

        jLabel14.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel14.setText("Grid worker count:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel14, gridBagConstraints);

        lWorkerCount.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lWorkerCount.setText("...");
        lWorkerCount.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lWorkerCount, gridBagConstraints);

        jLabel15.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel15.setText("Grid work request queue size:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(jLabel15, gridBagConstraints);

        lQueueSize.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        lQueueSize.setText("...");
        lQueueSize.setForeground(java.awt.SystemColor.activeCaption);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pServer.add(lQueueSize, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        getContentPane().add(pServer, gridBagConstraints);

        pWorkers.setLayout(new java.awt.GridBagLayout());

        pWorkers.setBorder(new javax.swing.border.TitledBorder("JCGrid Workers"));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        pWorkers.add(sPane, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pWorkers, gridBagConstraints);

        bRefresh.setText("Refresh");
        bRefresh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bRefreshActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        getContentPane().add(bRefresh, gridBagConstraints);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-400)/2, (screenSize.height-300)/2, 400, 300);
    }//GEN-END:initComponents

    private void bRefreshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bRefreshActionPerformed
        this.refreshInfo(adminConfig);
    }//GEN-LAST:event_bRefreshActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bRefresh;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel lHostName;
    private javax.swing.JLabel lHostPort;
    private javax.swing.JLabel lPing;
    private javax.swing.JLabel lQueueSize;
    private javax.swing.JLabel lSessionName;
    private javax.swing.JLabel lWorkerCount;
    private javax.swing.JPanel pServer;
    private javax.swing.JPanel pWorkers;
    private javax.swing.JScrollPane sPane;
    // End of variables declaration//GEN-END:variables
    
}
