/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.gui;


import java.io.*;
import java.text.*;
import java.util.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.table.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.admin.*;
import org.homedns.dade.jcgrid.message.*;

public class guiJCGridConfig extends javax.swing.JDialog {
    private final static String className=guiJCGridConfig.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private static final long serialVersionUID=1L;

    private GridConfig gridConfig;

    public guiJCGridConfig(GridConfig cfg, java.awt.Frame parent, boolean modal) {
        super(parent,modal);

        initComponents();

        this.refreshConfig(cfg);
    }

    public void refreshConfig(GridConfig cfg) {
        gridConfig=cfg;
        
        tServerName.setText(gridConfig.getServerAddress());

        tClientPort.setText(Integer.toString(gridConfig.getServerClientPort()));
        tWorkerPort.setText(Integer.toString(gridConfig.getServerWorkerPort()));
        tAdminPort.setText(Integer.toString(gridConfig.getServerAdminPort()));
        cbUseCompression.setSelected(gridConfig.getUseCompression());
        cbUseEnc.setSelected(gridConfig.getUseSecureConnection());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        pSettings = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        tServerName = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        tClientPort = new javax.swing.JTextField();
        bDefaultClientPort = new javax.swing.JButton();
        jLabel12 = new javax.swing.JLabel();
        tWorkerPort = new javax.swing.JTextField();
        bDefaultWorkerPort = new javax.swing.JButton();
        jLabel13 = new javax.swing.JLabel();
        tAdminPort = new javax.swing.JTextField();
        bDefaultAdminPort = new javax.swing.JButton();
        jLabel14 = new javax.swing.JLabel();
        cbUseEnc = new javax.swing.JCheckBox();
        bDefaultUseEnc = new javax.swing.JButton();
        jLabel15 = new javax.swing.JLabel();
        cbUseCompression = new javax.swing.JCheckBox();
        bDefaultUseCompression = new javax.swing.JButton();
        jLabel16 = new javax.swing.JLabel();
        bDefaultPwd = new javax.swing.JButton();
        pfPwd = new javax.swing.JPasswordField();
        bApply = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridBagLayout());

        setTitle("JCGrid Settings");
        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        pSettings.setLayout(new java.awt.GridBagLayout());

        pSettings.setBorder(new javax.swing.border.TitledBorder("JCGrid Settings"));
        jLabel10.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel10.setText("Host name:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel10, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(tServerName, gridBagConstraints);

        jLabel11.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel11.setText("Client port:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel11, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(tClientPort, gridBagConstraints);

        bDefaultClientPort.setText("Default");
        bDefaultClientPort.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultClientPortActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultClientPort, gridBagConstraints);

        jLabel12.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel12.setText("Worker port:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel12, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(tWorkerPort, gridBagConstraints);

        bDefaultWorkerPort.setText("Default");
        bDefaultWorkerPort.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultWorkerPortActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultWorkerPort, gridBagConstraints);

        jLabel13.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel13.setText("Admin port:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel13, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(tAdminPort, gridBagConstraints);

        bDefaultAdminPort.setText("Default");
        bDefaultAdminPort.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultAdminPortActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultAdminPort, gridBagConstraints);

        jLabel14.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel14.setText("Enable encryption:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel14, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(cbUseEnc, gridBagConstraints);

        bDefaultUseEnc.setText("Default");
        bDefaultUseEnc.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultUseEncActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultUseEnc, gridBagConstraints);

        jLabel15.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel15.setText("Enable compression:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel15, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(cbUseCompression, gridBagConstraints);

        bDefaultUseCompression.setText("Default");
        bDefaultUseCompression.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bDefaultUseCompressionActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultUseCompression, gridBagConstraints);

        jLabel16.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        jLabel16.setText("Server password:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(jLabel16, gridBagConstraints);

        bDefaultPwd.setText("Default");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bDefaultPwd, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(pfPwd, gridBagConstraints);

        bApply.setText("Apply");
        bApply.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bApplyActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
        pSettings.add(bApply, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        getContentPane().add(pSettings, gridBagConstraints);

        setBounds(10, 10, 400, 300);
    }//GEN-END:initComponents

    private void bApplyActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bApplyActionPerformed
        // Check settings

        String hostName=tServerName.getText().trim();
        if(hostName.equals("")) {
            JOptionPane.showMessageDialog(this,"You have to set the JCGrid Server host name.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        int clientPort=-1;
        try {
            clientPort=Integer.parseInt(tClientPort.getText());
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(this,"Syntax error in the JCGrid Server port.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if(clientPort<=1024) {
            JOptionPane.showMessageDialog(this,"JCGrid client port must be greater than 1024.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        int workerPort=-1;
        try {
            workerPort=Integer.parseInt(tWorkerPort.getText());
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(this,"Syntax error in the JCGrid worker port.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if(workerPort<=1024) {
            JOptionPane.showMessageDialog(this,"JCGrid worker port must be greater than 1024.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }

        int adminPort=-1;
        try {
            adminPort=Integer.parseInt(tAdminPort.getText());
        } catch(Exception ex) {
            JOptionPane.showMessageDialog(this,"Syntax error in the JCGrid admin. port.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        if(adminPort<=1024) {
            JOptionPane.showMessageDialog(this,"JCGrid admin. port must be greater than 1024.",
                "Settings error",JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        boolean useComp=cbUseCompression.isSelected();
        boolean useEnc=cbUseEnc.isSelected();
        
        // Applying settings
        
        gridConfig.setServerAddress(hostName);
        gridConfig.setServerClientPort(clientPort);
        gridConfig.setServerWorkerPort(workerPort);
        gridConfig.setServerAdminPort(adminPort);
        gridConfig.setUseCompression(useComp);
        gridConfig.setUseSecureConnection(useEnc);
    }//GEN-LAST:event_bApplyActionPerformed

    private void bDefaultUseCompressionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultUseCompressionActionPerformed
        cbUseCompression.setSelected(false);
    }//GEN-LAST:event_bDefaultUseCompressionActionPerformed

    private void bDefaultUseEncActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultUseEncActionPerformed
        cbUseEnc.setSelected(false);
    }//GEN-LAST:event_bDefaultUseEncActionPerformed

    private void bDefaultAdminPortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultAdminPortActionPerformed
        tAdminPort.setText(Integer.toString(GridConfig.DEFAULT_ADMIN_PORT));
    }//GEN-LAST:event_bDefaultAdminPortActionPerformed

    private void bDefaultWorkerPortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultWorkerPortActionPerformed
        tWorkerPort.setText(Integer.toString(GridConfig.DEFAULT_WORKER_PORT));
    }//GEN-LAST:event_bDefaultWorkerPortActionPerformed

    private void bDefaultClientPortActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDefaultClientPortActionPerformed
        tClientPort.setText(Integer.toString(GridConfig.DEFAULT_CLIENT_PORT));
    }//GEN-LAST:event_bDefaultClientPortActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bApply;
    private javax.swing.JButton bDefaultAdminPort;
    private javax.swing.JButton bDefaultClientPort;
    private javax.swing.JButton bDefaultPwd;
    private javax.swing.JButton bDefaultUseCompression;
    private javax.swing.JButton bDefaultUseEnc;
    private javax.swing.JButton bDefaultWorkerPort;
    private javax.swing.JCheckBox cbUseCompression;
    private javax.swing.JCheckBox cbUseEnc;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JPanel pSettings;
    private javax.swing.JPasswordField pfPwd;
    private javax.swing.JTextField tAdminPort;
    private javax.swing.JTextField tClientPort;
    private javax.swing.JTextField tServerName;
    private javax.swing.JTextField tWorkerPort;
    // End of variables declaration//GEN-END:variables
    
}
