/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.gui;

import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;

import org.apache.log4j.*;

public class guiRenderingPannel extends javax.swing.JPanel {
    private final static String className=guiRenderingPannel.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private static final long serialVersionUID=1L;

    private BufferedImage bImage;
    private Graphics2D graph;

    private boolean enableCursor;
    private int cursorX;
    private int cursorY;
    
    public guiRenderingPannel() {
        initComponents();
        
        enableCursor=false;
        cursorX=0;
        cursorY=0;
    }

    public synchronized void setCursorEnable(boolean v) {
        enableCursor=v;
    }

    public double getCursorX() {
        return cursorX/(double)this.getWidth();
    }

    public double getCursorY() {
        return 1.0-cursorY/(double)this.getHeight();
    }

    public synchronized void initImage() {
        int w=this.getSize().width;
        int h=this.getSize().height;

        if((w<=0) || (h<=0)) {
            bImage=null;
            graph=null;
            return;
        }
            
        bImage=new BufferedImage(w,h,BufferedImage.TYPE_3BYTE_BGR);
        graph=bImage.createGraphics();

        this.clearImage();
    }
    
    public Graphics2D getGraphics2D() {
        return graph;
    }
    
    public synchronized void saveImage(String imgName) {
        int idx=imgName.lastIndexOf('.');
        if(idx==-1)
            JOptionPane.showMessageDialog(this,"Missing image file extension",
                    "IO error",JOptionPane.ERROR_MESSAGE);
        else {
            String outputFormat=imgName.substring(idx+1);
            
            String[] formats=ImageIO.getWriterFormatNames();
            boolean found=false;
            for(int i=0;i<formats.length;i++) {
                if(formats[i].equals(outputFormat)) {
                    found=true;
                    break;
                }
            }
            
            if(!found)
                JOptionPane.showMessageDialog(this,"The image format "+outputFormat+
                        " is not supported","IO error",JOptionPane.ERROR_MESSAGE);
            else {
                File f=new File(imgName);
                try {
                    ImageIO.write(bImage,outputFormat,f);
                } catch(Exception ex) {
                    log.warn("Error saving the image",ex);

                    JOptionPane.showMessageDialog(this,"Error saving the image",
                        "IO error",JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }
    
    public synchronized void resizeImage(int width, int height) {
        if((width==bImage.getWidth()) &&
            (height==bImage.getHeight()))
            return;
        
        bImage=new BufferedImage(width,height,BufferedImage.TYPE_3BYTE_BGR);
        graph=bImage.createGraphics();

        this.clearImage();
    }

    public synchronized void clearImage() {
        int w=this.getSize().width;
        int h=this.getSize().height;

        if((w<=0) || (h<=0)) {
            bImage=null;
            graph=null;
            return;
        }

        for(int y=0;y<h;y+=10) {
            for(int x=0;x<w;x+=10) {
                if(((x/10+y/10) % 2)==0)
                    graph.setColor(Color.BLACK);
                else
                    graph.setColor(Color.WHITE);
                graph.fillRect(x,y,10,10);
            }
        }

        this.repaint();
    }

    public synchronized void paintComponent(Graphics g) {
        super.paintComponent(g);

        // Retry to inizilize image

        if(bImage==null)
            this.initImage();

        // Repaint image
        
        if(bImage!=null)
            g.drawImage(bImage,0,0,Color.BLACK,null);

        if(enableCursor) {
            g.setColor(Color.YELLOW);
            g.setXORMode(Color.BLUE);
            g.drawLine(cursorX-5,cursorY,cursorX+5,cursorY);
            g.drawLine(cursorX,cursorY-5,cursorX,cursorY+5);
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        setLayout(new java.awt.BorderLayout());

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });

    }//GEN-END:initComponents

    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        cursorX=evt.getX();
        cursorY=evt.getY();

        this.repaint();
    }//GEN-LAST:event_formMouseClicked
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
