/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.server;

import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.util.*;

public class AdminHandlerThread extends HandlerThread {
    public AdminHandlerThread(GridServer server, Socket socket) throws IOException {
        super(GridNodeConfig.TYPE_ADMIN,server,socket);

        if(log.isDebugEnabled()) log.debug("Start AdminHandlerThread("+server+","+socket+")");
        if(log.isDebugEnabled()) log.debug("End AdminHandlerThread()");
    }

    protected void handleMsg(GridMessage msg) throws Exception {
        if(log.isDebugEnabled()) log.debug("Start handleMsg()");

        if(msg instanceof GridMessageAdminGetWorkerStats) {
            if(log.isDebugEnabled()) log.debug("  Received work request AdminGetWorkerStats");

            HandlerThreads whs=gridServer.getWorkerHandlers();
            GridMessageAdminWorkerStats ws=new GridMessageAdminWorkerStats();
            for(Iterator i=whs.getSessionNames().iterator();i.hasNext();) {
                String name=(String)i.next();
                
                WorkerHandlerThread wht=(WorkerHandlerThread)
                    whs.getConnectedHandler(name);
                if(wht!=null) {
                    if(log.isDebugEnabled()) log.debug("    Session name: "+wht.getSessionName());

                    WorkerStats wstat=wht.getStats();
                    
                    if(wstat!=null)
                        ws.addStats(wstat);
                }
            }
            
            handlerChannel.send(ws);
        } else if(msg instanceof GridMessageAdminGetGridStats) {
            if(log.isDebugEnabled()) log.debug("  Received work request AdminGetGridStats");

            int workerCount=gridServer.getWorkerHandlers().getConnectedHandlerCount();
            int queueSize=gridServer.getRequestQueueSize();
            
            GridStats s=new GridStats(workerCount,queueSize);
            GridMessageAdminGridStats gs=new GridMessageAdminGridStats(s);
            handlerChannel.send(gs);
        } else
            log.warn("Unknown message in AdminHandlerThread.handleMsg(): "+msg);
    }
}
