/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.server;

import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.vfs.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.util.*;

public class ClientHandlerThread extends HandlerThread {
    public ClientHandlerThread(GridServer server, Socket socket) throws IOException {
        super(GridNodeConfig.TYPE_CLIENT,server,socket);

        if(log.isDebugEnabled()) log.debug("Start ClientHandlerThread("+server+","+socket+")");
        if(log.isDebugEnabled()) log.debug("End ClientHandlerThread()");
    }

    public void pushWorkResult(WorkResult wr) throws Exception {
        if(log.isDebugEnabled()) log.debug("Start pushWorkResult("+wr+")");

        // In order to avoid collision between more workers calling this method
        
        synchronized(handlerChannel) {
            handlerChannel.send(new GridMessageWorkResult(wr));
        }

        if(log.isDebugEnabled()) log.debug("End pushWorkResult()");
    }

    protected void handleMsg(GridMessage msg) throws Exception {
        if(log.isDebugEnabled()) log.debug("Start handleMsg()");

        if(msg instanceof GridMessageWorkRequest) {
            WorkRequest wr=((GridMessageWorkRequest)msg).getWorkRequest();

            // Reset the session name (security)

            wr.setSessionName(sessionName);

            if(log.isDebugEnabled()) log.debug("  Received work request: "+wr);

            gridServer.pushWorkRequest(wr);
        } else if(msg instanceof GridMessageVFSSessionBegin) {
            if(log.isDebugEnabled()) log.debug("  Received VFS session begin");

            if(gridServer.getNodeConfig().getGridConfig().getUseVFS()) {
                vfsSession clientSession=((GridMessageVFSSessionBegin)msg).getVFSSession();

                // Align sessions

                gridServer.getVFSSessionPool().syncVFSSession(
                    sessionName,handlerChannel,clientSession);

                // End file exchange

                handlerChannel.send(new GridMessageVFSSessionEnd());
            } else
                log.warn("Received VFS session begin even if VFS is disabled (session name: "+this.getSessionName()+")");
        } else
            log.warn("Unknown message in ClientHandlerThread.handleMsg(): "+msg);
        
        if(log.isDebugEnabled()) log.debug("End handleMsg()");
    }
    
    protected void handleDisconnection() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start handleDisconnection()");

        // Remove any pending work

        gridServer.removePendingWorkRequests(sessionName);

        if(gridServer.getNodeConfig().getGridConfig().getUseVFS())
            gridServer.getVFSSessionPool().unlockVFSSession(sessionName);
        
        if(log.isDebugEnabled()) log.debug("End handleDisconnection()");
    }
}
