/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.server;

import java.io.*;
import java.util.*;

import org.homedns.dade.jcgrid.*;

public class GridNodeServerConfig extends GridNodeConfig implements Cloneable {
    public final static int NETWORK_TIMEOUT=30*1000;

    private int maxConnections;
    private long maxCacheSize;

    public GridNodeServerConfig(Properties prop) {
        super(prop);

        maxConnections=Integer.parseInt(prop.getProperty(
            "grid.server.maxconnections","32"));
        maxCacheSize=Long.parseLong(prop.getProperty(
            "grid.server.cache.size",Long.toString(64*1024*1024)));
    }

    public GridNodeServerConfig() {
        super(GridNodeConfig.TYPE_SERVER);

        maxConnections=32;
        maxCacheSize=64*1024*1024;
    }

    public Object clone() {
        GridNodeServerConfig cfg=(GridNodeServerConfig)super.clone();
        cfg.maxConnections=maxConnections;
        cfg.maxCacheSize=maxCacheSize;

        return cfg;
    }

    public Properties toProperties() {
        Properties prop=super.toProperties();

        prop.put("grid.server.maxconnections",Integer.toString(maxConnections));
        prop.put("grid.server.cache.size",Long.toString(maxCacheSize));

        return prop;
    }

    //------------------------------- Get & Set --------------------------------

    public int getMaxConnections() {        
        return maxConnections;
    }

    public void setMaxConnections(int serverMaxConnections) {
        maxConnections=serverMaxConnections;
    }

    public long getMaxCacheSize() {
        return maxCacheSize;
    }

    public void setMaxCacheSize(long serverMaxCacheSize) {
        maxCacheSize=serverMaxCacheSize;
    }
}
