/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.server;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.util.*;

public class MainDemon extends Thread {
    protected final static String className=MainDemon.class.getName();
    protected static Logger log=Logger.getLogger(className);
    protected static Logger logDetail=Logger.getLogger("DETAIL."+className);

    private GridServer gridServer;
    private ServerSocket socket;
    private String handlerType;

    public MainDemon(GridServer server, String type) throws Exception {
        if(log.isDebugEnabled()) log.debug("Start MainDemon("+server+")");

        gridServer=server;
        handlerType=type;
        
        int port;
        if(handlerType.equals(GridNodeConfig.TYPE_CLIENT))
            port=gridServer.getNodeConfig().getGridConfig().getServerClientPort();
        else if(handlerType.equals(GridNodeConfig.TYPE_WORKER))
            port=gridServer.getNodeConfig().getGridConfig().getServerWorkerPort();
        else if(handlerType.equals(GridNodeConfig.TYPE_ADMIN))
            port=gridServer.getNodeConfig().getGridConfig().getServerAdminPort();
        else
            throw new Exception("Unknown handler type: "+handlerType);
        
        if(server.getNodeConfig().getGridConfig().getUseSecureConnection()) {
            SSLServerSocketFactory sslSrvFact=(SSLServerSocketFactory)
                SSLServerSocketFactory.getDefault();
            socket=sslSrvFact.createServerSocket(port);
        } else
            socket=new ServerSocket(port);

        if(log.isDebugEnabled()) log.debug("End MainDemon()");
    }

    public void run() {
        if(log.isDebugEnabled()) log.debug("Start run()");

        while(!this.isInterrupted()) {
            try {
                // Accept worker connection

                Socket hSocket=socket.accept();
                SocketAddress sa=hSocket.getRemoteSocketAddress();
                if((sa!=null) && (sa instanceof InetSocketAddress)) {
                    InetSocketAddress isa=(InetSocketAddress)sa;
                    log.warn("Connection from "+isa.getAddress()+":"+isa.getPort());
                } else
                    log.warn("Connection from unknow");

                if(handlerType.equals(GridNodeConfig.TYPE_CLIENT)) {
                    // Check the number of connection

                    int count=gridServer.getClientHandlers().getConnectedHandlerCount();

                    long maxConn=((GridNodeServerConfig)gridServer.getNodeConfig()).getMaxConnections();
                    if(count>maxConn) {
                        log.warn("Too many connections");
                        break;
                    }
                }

                // Spawn a thread to handle worker connection

                HandlerThread ht;
                if(handlerType.equals(GridNodeConfig.TYPE_CLIENT))
                    ht=new ClientHandlerThread(gridServer,hSocket);
                else if(handlerType.equals(GridNodeConfig.TYPE_WORKER))
                    ht=new WorkerHandlerThread(gridServer,hSocket);
                else if(handlerType.equals(GridNodeConfig.TYPE_ADMIN))
                    ht=new AdminHandlerThread(gridServer,hSocket);
                else
                    throw new Exception("Unknown handler type: "+handlerType);
                ht.start();
            } catch(Exception ex) {
                log.warn("Error in MainDemon.run()",ex);

                try {
                    this.sleep(500);
                } catch(InterruptedException iex) {
                    break;
                }
            }
        }
        
        try {
            socket.close();
        } catch(Exception ex) { }

        if(log.isDebugEnabled()) log.debug("End run()");
   }
}
