/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.server;

import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.vfs.*;
import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.util.*;

public class WorkerHandlerThread extends HandlerThread {
    private WorkerStats workerStats;

    public WorkerHandlerThread(GridServer server, Socket socket) throws IOException {
        super(GridNodeConfig.TYPE_WORKER,server,socket);

        if(log.isDebugEnabled()) log.debug("Start WorkerHandlerThread("+server+","+socket+")");
        
        workerStats=null;
        
        if(log.isDebugEnabled()) log.debug("End WorkerHandlerThread()");
    }

    public WorkerStats getStats() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getStats()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getStats("+workerStats+")");

        return workerStats;
    }
    
    protected void handleConnection() throws Exception {
        if(log.isDebugEnabled()) log.debug("Start handleConnection()");

        // Now the name of the worker is available
        
        workerStats=new WorkerStats(this.getSessionName());

        try {
            // Wait some work to do

            for(;;) {
                WorkRequest wr=gridServer.popWorkRequest(60*1000);
                if(wr==null) {
                    // Send a ping message in order to check if the worker
                    // is still connected

                    if(log.isDebugEnabled()) log.debug("  Sending Ping...");
                    handlerChannel.send(new GridMessagePing());

                    if(log.isDebugEnabled()) log.debug("  Waiting PingAck...");
                    GridMessage msg=handlerChannel.recv(30*1000);

                    if((msg==null) || (!(msg instanceof GridMessagePingAck)))
                        break;
                    else
                        continue;
                }
                
                WorkResult wres=null;
                try {
                    String sessionName=wr.getSessionName();
                    if(log.isDebugEnabled()) log.debug("  Received work request: "+wr);
                    
                    workerStats.workBegin(sessionName);

                    // Send the work request to the worker

                    handlerChannel.send(new GridMessageWorkRequest(wr));

                    if(gridServer.getNodeConfig().getGridConfig().getUseVFS()) {
                        // Align sessions

                        vfsSession session=gridServer.getVFSSessionPool().getVFSSession(sessionName);
                        session.syncVFSSession(handlerChannel);
                    }

                    // Wait for the answer

                    GridMessageWorkResult gmwr=(GridMessageWorkResult)handlerChannel.recv();
                    wres=gmwr.getWorkResult();

                    // Reset the user name (security)

                    wres.setSessionName(sessionName);

                    if(log.isDebugEnabled()) log.debug("  Received work result: "+wres);
                    
                    workerStats.workEnd(wres);
                } finally {                    
                    if(wres==null) {
                        // Failed to do the work. Reinsert the WorkRequest in the
                        // pending work queue

                        gridServer.pushWorkRequest(wr);
                    } else {
                        // Work done

                        gridServer.pushWorkResult(wres);
                    }
                }
            }
        } catch(EOFException eofex) {
            // Worker disconnected

            if(log.isDebugEnabled()) log.debug("  Socket EOF");
        }
        
        if(log.isDebugEnabled()) log.debug("End handleConnection()");
    }
}
