/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.util;

import java.io.*;

import org.apache.log4j.*;

public class ConsoleCmd extends Object {
    private final static String className=ConsoleCmd.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    //--------------------------------------------------------------------------
    
    private class WritterThread extends Thread {
        private OutputStream os;
        private byte[] data;

        public WritterThread(OutputStream osVal, byte[] dataVal) {
            os=osVal;
            data=dataVal;
        }

        public void run() {
            try {
                if(data!=null) {
                    os.write(data);
                    os.flush();
                }

                os.close();
            } catch(Throwable ex) {
                log.warn("Exception in ConsoleCmd.WritterThread.run()",ex);
            }
        }
    }
    
    private class ReaderThread extends Thread {
        private InputStream is;
        private byte[] data;

        public ReaderThread(InputStream isVal) {
            is=new BufferedInputStream(isVal);
        }

        public void run() {
            try {
                data=ByteVector.readAll(is);
            } catch(Throwable ex) {
                log.warn("Exception in ConsoleCmd.ReaderThread.run()",ex);
            }
        }
        
        public byte[] getData() {
            return data;
        }
    }

    //--------------------------------------------------------------------------
    
    private String[] cmd;
    private Process proc;
    private byte[] stdin;
    private byte[] stdout;
    private byte[] stderr;
    private File curDir;

    public ConsoleCmd(String[] cmdVal, byte[] stdinVal, String dir) {
        stdin=stdinVal;
        cmd=cmdVal;
        proc=null;
        
        if(dir==null)
            curDir=null;
        else
            curDir=new File(dir);
    }

    public ConsoleCmd(String[] cmdVal, byte[] stdinVal) {
        this(cmdVal,stdinVal,null);
    }
  
    public ConsoleCmd(String[] cmdVal) {
        this(cmdVal,null,null);
    }
    
    public String getCmdLine() {
        StringBuffer sb=new StringBuffer();

        boolean first=true;
        for(int i=0;i<cmd.length;i++) {
            if(first)
                first=false;
            else
                sb.append(" ");

            sb.append(cmd[i]);
        }
        
        return sb.toString();
    }
    
    public byte[] getStdOut() {
        return stdout;
    }

    public byte[] getStdErr() {
        return stderr;
    }
    
    public int run() throws IOException, InterruptedException {
        if(curDir==null)
            proc=Runtime.getRuntime().exec(cmd);
        else
            proc=Runtime.getRuntime().exec(cmd,null,curDir);

        OutputStream si=proc.getOutputStream();
        InputStream so=proc.getInputStream();
        InputStream se=proc.getErrorStream();
 
        // Thread stdin

        WritterThread it=new WritterThread(si,stdin);
        it.start();

        // Thread stdout
        
        ReaderThread ot=new ReaderThread(so);
        ot.start();

        // Thread stderr
        
        ReaderThread et=new ReaderThread(se);
        et.start();
        
        
        // Wait execution end
        
        int exitVal=proc.waitFor();
        it.join();
        ot.join();
        et.join();

        stdout=ot.getData();
        stderr=et.getData();

        return exitVal;
    }
}
