/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.util;

import java.io.*;
import java.net.*;
import java.util.zip.*;

import org.homedns.dade.jcgrid.message.*;

public class GridMessageFlatChannel implements GridMessageChannel {
    private Socket socket;
    private ObjectInputStream objIn;
    private ObjectOutputStream objOut;

    public GridMessageFlatChannel(Socket s, boolean isServer) throws IOException {
        socket=s;

        InputStream is=socket.getInputStream();
        OutputStream os=socket.getOutputStream();

        if(isServer) {
            objIn=new ObjectInputStream(is);
            objOut=new ObjectOutputStream(os);
        } else {
            objOut=new ObjectOutputStream(os);
            objIn=new ObjectInputStream(is);
        }
    }
    
    public GridMessage recv() throws IOException, ClassNotFoundException {
        GridMessage msg=(GridMessage)objIn.readObject();
        
        return msg;
    }

    public GridMessage recv(int timeout) throws IOException, ClassNotFoundException {
        socket.setSoTimeout(timeout);
        GridMessage msg=(GridMessage)objIn.readObject();
        socket.setSoTimeout(0);
        
        return msg;
    }

    public void send(GridMessage msg) throws IOException, ClassNotFoundException  {
        objOut.writeObject(msg);
        objOut.flush();

        // In order to avoid a well know out of memory problem
        // of ObjectOutputStream class

        objOut.reset();
    }
    
    public void close() throws IOException {
        objIn.close();
        objOut.close();
        socket.close();
    }
}
