/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.util;

import java.io.*;
import java.util.*;

public class Queue {
    protected Vector elements;

    public Queue() {
        elements=new Vector();
    }
    
    public Object pop()
        throws InterruptedException {
        return this.pop(null);
    }
    
    public Object pop(long timeout)
        throws InterruptedException {
        return this.pop(null,timeout);
    }

    public Object pop(FilterQueueObject fqo)
        throws InterruptedException {
        return this.pop(fqo,0);
    }
   
    public Object pop(FilterQueueObject fqo, long timeout)
        throws InterruptedException {
        Object qo=null;
        boolean found=false;
        boolean checkAndExit=false;
        
        synchronized(elements) {
            while(!found) {
                for(int i=0;i<elements.size();i++) {
                    Object o=(Object)elements.elementAt(i);

                    if((fqo==null) || (fqo.isValid(o))) {
                        elements.removeElementAt(i);
                        qo=o;
                        found=true;
                        break;
                    }
                }

                if(checkAndExit)
                    break;

                if(!found) {                
                    elements.wait(timeout);
                 
                    if(timeout>0)
                        checkAndExit=true;
                }
            }
        }

        return qo;
    }
    
    public void push(Object qo) {
        synchronized(elements) {
            elements.add(qo);
            elements.notifyAll();
        }
    }
    
    public int size() {
        synchronized(elements) {
            return elements.size();
        }
    }
}