/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker;

import java.io.*;
import java.net.*;
import javax.net.ssl.*;

import org.apache.log4j.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.vfs.*;
import org.homedns.dade.jcgrid.util.*;

public class GridWorker extends GridNodeGeneric {
    private final static String className=GridWorker.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);
    
    private Worker worker;
    private WorkerThread wt;
    private long maxCacheSize;

    private vfsSessionPool sessions;
    private GridWorkerFeedback feedback;

    public GridWorker() {
        super(new GridNodeWorkerConfig());

        if(log.isDebugEnabled()) log.debug("Start GridWorker()");

        worker=null;
        wt=null;
        maxCacheSize=64*1024*1024;

        sessions=null;
        feedback=null;

        if(log.isDebugEnabled()) log.debug("End GridWorker()");
    }

    //----------------------------- Start & Stop -------------------------------

    public void start() throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start start()");

        super.start();
        
        if(this.getNodeConfig().getGridConfig().getUseVFS()) {
            sessions=new vfsSessionPool(super.getNodeConfig().getWorkingDir(),maxCacheSize);
            sessions.start();
            log.warn("VFS Session cache size: "+sessions.getSize()/(1024*1024)+"/"+
                maxCacheSize/(1024*1024)+"MB");
        } else
            sessions=null;

        wt=new WorkerThread(this);
        wt.setWorkerFeedback(feedback);
        wt.start();

        if(logDetail.isDebugEnabled()) logDetail.debug("End start()");
    }

    public void waitShutdown() throws InterruptedException {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start waitShutdown()");

        wt.join();

        if(logDetail.isDebugEnabled()) logDetail.debug("End waitShutdown()");
    }

    public void stop() throws Exception {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start stop()");

        wt.interrupt();
        try {
            wt.join();
        } catch(InterruptedException iex) { }

        super.stop();

        if(logDetail.isDebugEnabled()) logDetail.debug("End stop()");
    }

    //------------------------------- Get & Set --------------------------------

    public void setWorkerFeedback(GridWorkerFeedback f) {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start setWorkerFeedback("+f+")");

        feedback=f;
        if(wt!=null)
            wt.setWorkerFeedback(f);

        if(logDetail.isDebugEnabled()) logDetail.debug("End setWorkerFeedback()");
    }

    public vfsSessionPool getVFSSessionPool() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getVFSSessionPool()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getVFSSessionPool("+sessions+")");
        
        return sessions;
    }

    public Worker getWorker() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getWorker()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getWorker("+worker+")");
        
        return worker;
    }

    public void setWorker(Worker w) {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start setWorker("+w+")");

        worker=w;

        if(logDetail.isDebugEnabled()) logDetail.debug("End setWorker()");
    }

    public long getMaxCacheSize() {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start getMaxCacheSize()");
        if(logDetail.isDebugEnabled()) logDetail.debug("End getMaxCacheSize("+maxCacheSize+")");
        
        return maxCacheSize;
    }

    public void setMaxCacheSize(long serverMaxCacheSize) {
        if(logDetail.isDebugEnabled()) logDetail.debug("Start setMaxCacheSize("+serverMaxCacheSize+")");

        maxCacheSize=serverMaxCacheSize;

        if(logDetail.isDebugEnabled()) logDetail.debug("End setMaxCacheSize()");
    }
}
