/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.mandel;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;

import org.homedns.dade.jcgrid.message.*;
import org.homedns.dade.jcgrid.client.*;

public class MandelRenderingClient extends Thread {
    private GridNodeClientConfig clientCfg;
    private MandelRenderingClientFeedback clientFeedback;

    private MandelWorkRequest mandelWReq;

    private MandelRenderingFrame frame;
    
    public MandelRenderingClient(GridNodeClientConfig cfg,
        MandelRenderingClientFeedback feedback,
        MandelWorkRequest req) {
        clientCfg=cfg;
        clientFeedback=feedback;

        mandelWReq=req;

        frame=new MandelRenderingFrame(req.getXStep(),req.getYStep(),req.getMaxIter());
    }

    public MandelRenderingFrame getFrame() {
        return frame;
    }
    
    public void run() {
        try {
            // Start Client
            
            GridClient gc=new GridClient();
            gc.setNodeConfig(clientCfg);
            gc.start();

            long t1=System.currentTimeMillis();
            try {
                // Splitting the work
                
                MandelWorkRequest[] workList=mandelWReq.split();
                int fragCount=mandelWReq.fragmentPerFrame();

                clientFeedback.setProgressMaximum(0);
                clientFeedback.setProgressMaximum(2*workList.length-1);
                
                clientFeedback.renderingBegin();
                
                // Sending work requests

                for(int i=0;i<workList.length;i++) {
                    clientFeedback.setProgressValue(i);
                    clientFeedback.sendingFrgamentRequest(workList[i]);

                    gc.send(new GridMessageWorkRequest(workList[i]));

                    if(this.isInterrupted())
                        break;
                }

                // Receiving work results

                for(int i=0;i<workList.length;i++) {
                    clientFeedback.setProgressValue(i+workList.length);

                    GridMessageWorkResult gmwr=(GridMessageWorkResult)gc.recv();
                    MandelWorkResult mandelWRes=(MandelWorkResult)gmwr.getWorkResult();
                    int idx=mandelWRes.getRID();

                    frame.addFragmentResult(workList[idx],mandelWRes,frame);
                    clientFeedback.receivedFragmentResult(workList[idx],mandelWRes,frame);
                    
                    if(this.isInterrupted())
                        break;
                }
            } finally {
                long t2=System.currentTimeMillis();

                clientFeedback.setRenderingTime(mandelWReq,t2-t1);

                try {
                    gc.stop();
                } catch(Exception ex) { }
            }
        } catch(Exception ex) {
            clientFeedback.error("Error while doing the rendering",ex);
        }

        clientFeedback.renderingEnd();
    }
}
