/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.mandel;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;

public class MandelRenderingFrame {
    public final static int COLOR_TABLE_TYPE_RED=0;
    public final static int COLOR_TABLE_TYPE_MISC=1;

    private BufferedImage renderingFrameBuffer;
    private Graphics2D renderingFrameBufferG2D;

    private int colorTableType;
    private Color[] colorTable;

    public MandelRenderingFrame(int width, int height, int iter) {
        colorTableType=COLOR_TABLE_TYPE_RED;

        renderingFrameBuffer=new BufferedImage(
            width,height,BufferedImage.TYPE_3BYTE_BGR);
        renderingFrameBufferG2D=renderingFrameBuffer.createGraphics();
        
        for(int y=0;y<renderingFrameBuffer.getHeight();y+=10) {
            for(int x=0;x<renderingFrameBuffer.getWidth();x+=10) {
                if(((x/10+y/10) % 2)==0)
                    renderingFrameBufferG2D.setColor(Color.BLACK);
                else
                    renderingFrameBufferG2D.setColor(Color.WHITE);
                renderingFrameBufferG2D.fillRect(x,y,10,10);
            }
        }

        this.setMaxIter(iter);
    }
    
    public BufferedImage getBufferedImage() {
        return renderingFrameBuffer;
    }
    
    public Graphics2D getGraphics2D() {
        return renderingFrameBufferG2D;
    }

    public void setColorTableType(int type) {
        colorTableType=type;
    }

    public void setMaxIter(int maxIter) {
        colorTable=new Color[maxIter];

        float r;
        float g;
        float b;

        float dr;
        float dg;
        float db;

        if(colorTableType==COLOR_TABLE_TYPE_MISC) {
            r=1.0f;
            g=0.0f;
            b=0.0f;

            dr=0.23f;
            dg=0.37f;
            db=0.41f;
        } else {
            r=1.0f;
            g=0.0f;
            b=0.0f;

            dr=0.11f;
            dg=0.001f;
            db=0.001f;
        }
        
        for(int i=0;i<maxIter-1;i++) {
            colorTable[i]=new Color(r,g,b);

            r+=dr;
            g+=dg;
            b+=db;

            if(r>1.0)
                r-=1.0;
            if(g>1.0)
                g-=1.0;
            if(b>1.0)
                b-=1.0;
        }

        colorTable[maxIter-1]=Color.BLACK;
    }

    public void addFragmentResult(MandelWorkRequest req,
        MandelWorkResult res, MandelRenderingFrame frame) {
        int height=renderingFrameBuffer.getHeight()-1;

        for(int y=0;y<req.getYStep();y++) {
            for(int x=0;x<req.getXStep();x++) {
                renderingFrameBufferG2D.setColor(
                    colorTable[((res.getIter())[x][y]) % colorTable.length]);

                renderingFrameBufferG2D.fillRect(x+req.getXOffset(),
                    height-(y+req.getYOffset()),
                    1,1);
            }
        }
    }
}
