/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.mandel;

import java.math.*;

import org.homedns.dade.jcgrid.*;

public class MandelWorkRequest extends WorkRequest {
    private static final long serialVersionUID=1L;

    private short maxIter;

    private double xOrig;
    private double yOrig;

    private double dx;
    private double dy;
    
    private int xOffset;
    private int yOffset;
    
    private int xStep;
    private int yStep;

    public MandelWorkRequest(
        String nameVal, int idVal,
        short maxIterVal,
        double xOrigVal, double yOrigVal,
        double dxVal, double dyVal,
        int xStepVal, int yStepVal) {
        super(nameVal,idVal);

        maxIter=maxIterVal;

        xOrig=xOrigVal;
        yOrig=yOrigVal;

        dx=dxVal;
        dy=dyVal;
        
        xStep=xStepVal;
        yStep=yStepVal;
    }

    public int fragmentPerFrame() {
        int xCount=(int)Math.ceil(xStep/(double)MandelConfig.MANDELGRID_FRAG_SIZE);
        int yCount=(int)Math.ceil(yStep/(double)MandelConfig.MANDELGRID_FRAG_SIZE);

        return xCount*yCount;
    }

    public MandelWorkRequest[] split() {
        int xCount=(int)Math.ceil(xStep/(double)MandelConfig.MANDELGRID_FRAG_SIZE);
        int yCount=(int)Math.ceil(yStep/(double)MandelConfig.MANDELGRID_FRAG_SIZE);
        int count=xCount*yCount;
        
        MandelWorkRequest[] mfs=new MandelWorkRequest[count];

        int i=0;
        for(int y=0;y<yCount;y++) {
            int fh;
            if(y==yCount-1) {
                fh=yStep-y*MandelConfig.MANDELGRID_FRAG_SIZE;
            } else
                fh=MandelConfig.MANDELGRID_FRAG_SIZE;

            for(int x=0;x<xCount;x++) {
                int fw;
                if(x==xCount-1) {
                    fw=xStep-x*MandelConfig.MANDELGRID_FRAG_SIZE;
                } else
                    fw=MandelConfig.MANDELGRID_FRAG_SIZE;

                MandelWorkRequest mf=new MandelWorkRequest(
                    this.getSessionName(),i,
                    maxIter,
                    xOrig+dx*x*MandelConfig.MANDELGRID_FRAG_SIZE,
                    yOrig+dy*y*MandelConfig.MANDELGRID_FRAG_SIZE,
                    dx,dy,
                    fw,fh);
                mf.setXOffset(x*MandelConfig.MANDELGRID_FRAG_SIZE);
                mf.setYOffset(y*MandelConfig.MANDELGRID_FRAG_SIZE);

                mfs[i++]=mf;
            }
        }
        
        return mfs;
    }

    public String toString() {
        return "MandelWorkRequest["+maxIter+","+xOrig+","+yOrig+","+dx+","+dy+","+xOffset+","+yOffset+","+xStep+","+yStep+"]";
    }

    //------------------------------- Get & Set --------------------------------

    public void setMaxIter(short i) {
        maxIter=i;
    }
    
    public short getMaxIter() {
        return maxIter;
    }

    public void setXOrig(double xo) {
        xOrig=xo;
    }

    public double getXOrig() {
        return xOrig;
    }

    public void setYOrig(double yo) {
        yOrig=yo;
    }

    public double getYOrig() {
        return yOrig;
    }

    public void setDX(double d) {
        dx=d;
    }

    public double getDX() {
        return dx;
    }

    public void setDY(double d) {
        dy=d;
    }

    public double getDY() {
        return dy;
    }
    
    public void setXStep(int xs) {
        xStep=xs;
    }
    
    public int getXStep() {
        return xStep;
    }
    
    public void setYStep(int ys) {
        yStep=ys;
    }
    
    public int getYStep() {
        return yStep;
    }
    
    public void setXOffset(int xo) {
        xOffset=xo;
    }
    
    public int getXOffset() {
        return xOffset;
    }
    
    public void setYOffset(int yo) {
        yOffset=yo;
    }

    public int getYOffset() {
        return yOffset;
    }
}
