/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.mandel;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.worker.*;

public class MandelWorker implements Worker {
    public MandelWorker() {
    }

    private short mandelIterate(double x, double y, short maxIter) {
        double r=0.0;
        double i=0.0;
        double m=0.0;

        short iter=1;
        while((iter<maxIter) && (m<4.0)) {
            iter++;
            m=r*r-i*i;
            i=2.0*r*i+y;
            r=m+x;
        }

        return --iter;
    }

    public WorkResult doWork(WorkRequest work, String workDir) throws Exception {
        MandelWorkRequest mwr=(MandelWorkRequest)work;

        // Check request parameters
        
        if(mwr.getMaxIter()>MandelConfig.MANDELGRID_MAX_ITER)
            mwr.setMaxIter(MandelConfig.MANDELGRID_MAX_ITER);
        else if(mwr.getMaxIter()<MandelConfig.MANDELGRID_MIN_ITER)
            mwr.setMaxIter(MandelConfig.MANDELGRID_MIN_ITER);

        if(mwr.getXStep()>MandelConfig.MANDELGRID_FRAG_SIZE)
            mwr.setXStep(MandelConfig.MANDELGRID_FRAG_SIZE);
        else if(mwr.getXStep()<1)
            mwr.setYStep(1);

        if(mwr.getYStep()>MandelConfig.MANDELGRID_FRAG_SIZE)
            mwr.setYStep(MandelConfig.MANDELGRID_FRAG_SIZE);
        else if(mwr.getYStep()<1)
            mwr.setYStep(1);

        // Render fragment
        
        short[][] iter=new short[mwr.getXStep()][mwr.getYStep()];
        
        double y=mwr.getYOrig();
        for(int j=0;j<mwr.getYStep();j++) {
            double x=mwr.getXOrig();

            for(int i=0;i<mwr.getXStep();i++) {
                iter[i][j]=this.mandelIterate(x,y,mwr.getMaxIter());
                
                x+=mwr.getDX();
            }
            
            y+=mwr.getDY();
        }

        return new MandelWorkResult(mwr.getSessionName(),mwr.getRID(),iter);
    }
}
