/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.povray;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;

public class POVRenderingFrame {
    private BufferedImage renderingFrameBuffer;
    private Graphics2D renderingFrameBufferG2D;
    private int fragmentTotCount;

    private int fragmentCount;

    public POVRenderingFrame(int width, int height, int count) {
        renderingFrameBuffer=new BufferedImage(
            width,height,BufferedImage.TYPE_3BYTE_BGR);
        renderingFrameBufferG2D=renderingFrameBuffer.createGraphics();
        
        fragmentTotCount=count;
        fragmentCount=0;
        
        for(int y=0;y<renderingFrameBuffer.getHeight();y+=10) {
            for(int x=0;x<renderingFrameBuffer.getWidth();x+=10) {
                if(((x/10+y/10) % 2)==0)
                    renderingFrameBufferG2D.setColor(Color.BLACK);
                else
                    renderingFrameBufferG2D.setColor(Color.WHITE);
                renderingFrameBufferG2D.fillRect(x,y,10,10);
            }
        }
    }
    
    public BufferedImage getBufferedImage() {
        return renderingFrameBuffer;
    }
    
    public Graphics2D getGraphics2D() {
        return renderingFrameBufferG2D;
    }

    public void addFragmentResult(POVWorkRequest req, POVWorkResult res) {
        // Get the rendered fragment
        // I'm using this trick because POVRay file output
        // is not well formatted (height is wrong). POVRay requires
        // "wrong" height in order to support "+C" option.

        byte[] ppmData=res.getData();
        int ppmIdx=0;
        for(int j=0;j<3;ppmIdx++) {
            if(ppmData[ppmIdx]=='\n')
                j++;
        }

        for(int y=req.getStartRow();y<=req.getEndRow();y++) {
            for(int x=0;x<req.getWidth();x++) {
                int r=ppmData[ppmIdx++] & 0xff;                
                int g=ppmData[ppmIdx++] & 0xff;
                int b=ppmData[ppmIdx++] & 0xff;
                
                if((x>=req.getStartCol()) &&
                    (x<=req.getEndCol())) {
                    renderingFrameBufferG2D.setColor(new Color(r,g,b));
                    renderingFrameBufferG2D.fillRect(x,y,1,1);
                }
            }
        }

        fragmentCount++;
    }
    
    public boolean isComplete() {
        return fragmentCount>=fragmentTotCount;
    }
}
