/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.povray;

import java.math.*;
import java.util.*;

import org.homedns.dade.jcgrid.*;

public class POVWorkRequest extends WorkRequest {
    private static final long serialVersionUID=2L;

    private String iniFileName;
    private int width;
    private int height;
    private int startCol;
    private int endCol;
    private int startRow;
    private int endRow;

    private boolean animationEnabled;
    private int frame;

    public POVWorkRequest(String name, int id,
        String iniName,
        int w, int h,
        int sc, int ec,
        int sr, int er) {
        super(name,id);

        iniFileName=iniName;
        width=w;
        height=h;
        startCol=sc;
        endCol=ec;
        startRow=sr;
        endRow=er;

        animationEnabled=false;
        frame=0;
    }

    public int fragmentPerFrame(int wStep, int hStep) {
        int col=endCol-startCol+1;
        int row=endRow-startRow+1;

        int wCount=col/wStep;
        if(col % wStep!=0)
            wCount++;
        int hCount=row/hStep;
        if(row % hStep!=0)
            hCount++;
        
        return wCount*hCount;
    }
    
    public POVWorkRequest[] split(int wStep, int hStep) {
        return this.split(0,wStep,hStep);
    }

    public POVWorkRequest[] split(int RIDBase, int wStep, int hStep) {
        int col=endCol-startCol+1;
        int row=endRow-startRow+1;

        int wCount=col/wStep;
        if(col % wStep!=0)
            wCount++;
        int hCount=row/hStep;
        if(row % hStep!=0)
            hCount++;
        
        int count=wCount*hCount;

        POVWorkRequest[] res=new POVWorkRequest[count];

        int i=0;
        for(int h=0;h<hCount;h++) {
            int fh;
            if(h==hCount-1) {
                fh=row-h*hStep-1;
            } else
                fh=hStep-1;

            for(int w=0;w<wCount;w++) {
                int fw;
                if(w==wCount-1) {
                    fw=col-w*wStep-1;
                } else
                    fw=wStep-1;

                res[i]=new POVWorkRequest(
                    super.getSessionName(),
                    RIDBase+1+i,
                    iniFileName,
                    width,height,
                    w*wStep+startCol,w*wStep+startCol+fw,
                    h*hStep+startRow,h*hStep+startRow+fh);
                if(animationEnabled) {
                    res[i].setAnimationEnabled(true);
                    res[i].setFrame(frame);
                }
                    
                i++;
            }
        }

        return res;
    }

    public POVWorkRequest[] split(int startFrame, int endFrame,
        int wStep, int hStep) {
        return this.split(0,startFrame,endFrame,wStep,hStep);
    }
    
    public POVWorkRequest[] split(int RIDBase,
        int startFrame, int endFrame,
        int wStep, int hStep) {
        this.setAnimationEnabled(true);

        Vector v=new Vector();
        for(frame=startFrame;frame<=endFrame;frame++) {
            POVWorkRequest[] reqs=this.split(RIDBase,wStep,hStep);
            RIDBase+=reqs.length;
            v.addAll(Arrays.asList(reqs));
        }
        
        // While waiting for Java 1.5...

        POVWorkRequest[] reqs=new POVWorkRequest[v.size()];
        int j=0;
        for(Iterator i=v.iterator();i.hasNext();) 
            reqs[j++]=(POVWorkRequest)i.next();

        return reqs;
    }
    
    public String getINIFileName() {
        return iniFileName;
    }

    public int getWidth() {
        return width;
    }
    
    public int getHeight() {
        return height;
    }

    public int getStartCol() {  
        return startCol;
    }
    
    public int getEndCol() {  
        return endCol;
    }

    public int getStartRow() {  
        return startRow;
    }
    
    public int getEndRow() {  
        return endRow;
    }

    public void setAnimationEnabled(boolean v) {
        animationEnabled=v;
    }
    
    public boolean isAnimationEnabled() {
        return animationEnabled;
    }
    
    public void setFrame(int f) {
        frame=f;
    }
    
    public int getFrame() {
        return frame;
    }
}
