/*
 Copyright (C) 2004 David Bucciarelli (davibu@interfree.it)

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package org.homedns.dade.jcgrid.worker.impl.povray;

import java.io.*;

import org.homedns.dade.jcgrid.*;
import org.homedns.dade.jcgrid.worker.*;
import org.homedns.dade.jcgrid.util.*;

import org.apache.log4j.*;

public class POVWorker implements Worker {
    private final static String className=POVWorker.class.getName();
    private static Logger log=Logger.getLogger(className);
    private static Logger logDetail=Logger.getLogger("DETAIL."+className);

    public final static String TEMP_IMG_FILE="jcgrid-worker-tmp-img-file.ppm";
    
    private boolean usePVEngineEXE;
    
    public POVWorker(boolean useEXE) {
        usePVEngineEXE=useEXE;
    }

    public WorkResult doWork(WorkRequest work, String workDir) throws Exception {
        POVWorkRequest povwr=((POVWorkRequest)work);
        
        // Create command line

        String tmpFileName=workDir+File.separator+TEMP_IMG_FILE;
        String[] cmd;
        if(usePVEngineEXE) {
            if(povwr.isAnimationEnabled()) {
                String[] cmdAnim={
                    "pvengine.exe",
                    "/exit",
                    "/render",
                    "-V",
                    "-D",
                    "+W"+povwr.getWidth(),
                    "+H"+povwr.getHeight(),
                    "+SC"+(povwr.getStartCol()+1),
                    "+EC"+(povwr.getEndCol()+1),
                    "+SR"+(povwr.getStartRow()+1),
                    "+ER"+(povwr.getEndRow()+1),
                    "+FP",
                    "+O"+tmpFileName,
                    "+SF"+povwr.getFrame(),
                    "+EF"+povwr.getFrame(),
                    povwr.getINIFileName()
                };
                cmd=cmdAnim;
            } else {
                String[] cmdNoAnim={
                    "C:\\programmi\\povray-3.6\\bin\\pvengine.exe",
                    "/exit",
                    "/render",
                    "-V",
                    "-D",
                    "+W"+povwr.getWidth(),
                    "+H"+povwr.getHeight(),
                    "+SC"+(povwr.getStartCol()+1),
                    "+EC"+(povwr.getEndCol()+1),
                    "+SR"+(povwr.getStartRow()+1),
                    "+ER"+(povwr.getEndRow()+1),
                    "+FP",
                    "+O"+tmpFileName,
                    povwr.getINIFileName()
                };
                cmd=cmdNoAnim;
            }
        } else {
            if(povwr.isAnimationEnabled()) {
                String[] cmdAnim={
                    "povray",
                    "-V",
                    "-D",
                    "+W"+povwr.getWidth(),
                    "+H"+povwr.getHeight(),
                    "+SC"+(povwr.getStartCol()+1),
                    "+EC"+(povwr.getEndCol()+1),
                    "+SR"+(povwr.getStartRow()+1),
                    "+ER"+(povwr.getEndRow()+1),
                    "+FP",
                    "+O"+tmpFileName,
                    "+SF"+povwr.getFrame(),
                    "+EF"+povwr.getFrame(),
                    povwr.getINIFileName()
                };
                cmd=cmdAnim;
            } else {
                String[] cmdNoAnim={
                    "povray",
                    "-V",
                    "-D",
                    "+W"+povwr.getWidth(),
                    "+H"+povwr.getHeight(),
                    "+SC"+(povwr.getStartCol()+1),
                    "+EC"+(povwr.getEndCol()+1),
                    "+SR"+(povwr.getStartRow()+1),
                    "+ER"+(povwr.getEndRow()+1),
                    "+FP",
                    "+O"+tmpFileName,
                    povwr.getINIFileName()
                };
                cmd=cmdNoAnim;
            }
        }

        // Execute POVRay

        ConsoleCmd cc=new ConsoleCmd(cmd,null,workDir);

        if(cc.run()!=0) {
            StringBuffer sb=new StringBuffer();
            sb.append("Failed POVRay command execution:\n");

            sb.append("[WorkDir: \n");
            sb.append("<");
            sb.append(workDir);
            sb.append(">");

            sb.append("\n]\n[Cmd:\n");
            for(int i=0;i<cmd.length;i++) {
                sb.append("<");
                sb.append(cmd[i]);
                sb.append(">\n");
            }

            sb.append("]\n[stderr:\n");
            sb.append(new String(cc.getStdErr()));
            sb.append("\n]\n[stdout:\n");
            sb.append(new String(cc.getStdOut()));
            sb.append("\n]");

            log.warn(sb.toString());

            throw new Exception("Failed POVRay command execution");
        }

        if(cc.getStdErr().length>0)
            log.warn("Stderr execution in POVWorker.doWork(): "+new String(cc.getStdErr()));

        // Read temporary file. I can't use POVRay stdout because a bug of
        // java CreateProcess under Windows (0x0a are replaced with 0x0d0a)
        
        File tmpFile=new File(tmpFileName);
        FileInputStream fis=new FileInputStream(tmpFile);
        byte[] data=ByteVector.readAll(fis);
        fis.close();
        tmpFile.delete();

        // Create work result
        
        POVWorkResult res=new POVWorkResult(
            povwr.getSessionName(),povwr.getRID(),data,
            (povwr.getEndCol()-povwr.getStartCol()+1)*
                (povwr.getEndRow()-povwr.getStartRow()+1));
        
        return res;
    }
}
